@@ -1,6 +1,7 @@
 // SPDX-FileCopyrightText: 2017 Konstantinos Sideris <siderisk@auth.gr>
 // SPDX-FileCopyrightText: 2019 The nheko authors
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
+// SPDX-FileCopyrightText: 2022 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -43,8 +44,7 @@ public:
     QString avatarUrl(const QString &room_id, const QString &user_id);
 
     // presence
-    mtx::presence::PresenceState presenceState(const std::string &user_id);
-    std::string statusMessage(const std::string &user_id);
+    mtx::events::presence::Presence presence(const std::string &user_id);
 
     // user cache stores user keys
     std::map<std::string, std::optional<UserKeyCache>>
@@ -96,10 +96,12 @@ public:
         return getStateEvent<T>(txn, room_id, state_key);
     }
     template<typename T>
-    std::vector<mtx::events::StateEvent<T>> getStateEventsWithType(const std::string &room_id)
+    std::vector<mtx::events::StateEvent<T>>
+    getStateEventsWithType(const std::string &room_id,
+                           mtx::events::EventType type = mtx::events::state_content_to_type<T>)
     {
         auto txn = lmdb::txn::begin(env_, nullptr, MDB_RDONLY);
-        return getStateEventsWithType<T>(txn, room_id);
+        return getStateEventsWithType<T>(txn, room_id, type);
     }
 
     //! retrieve a specific event from account data
@@ -116,6 +118,7 @@ public:
                                                  std::size_t len        = 30);
     size_t memberCount(const std::string &room_id);
 
+    void updateState(const std::string &room, const mtx::responses::StateEvents &state);
     void saveState(const mtx::responses::Sync &res);
     bool isInitialized();
     bool isDatabaseReady() { return databaseReady_ && isInitialized(); }
@@ -215,6 +218,7 @@ public:
     uint64_t saveOldMessages(const std::string &room_id, const mtx::responses::Messages &res);
     void savePendingMessage(const std::string &room_id,
                             const mtx::events::collections::TimelineEvent &message);
+    std::vector<std::string> pendingEvents(const std::string &room_id);
     std::optional<mtx::events::collections::TimelineEvent>
     firstPendingMessage(const std::string &room_id);
     void removePendingStatus(const std::string &room_id, const std::string &txn_id);
@@ -494,13 +498,11 @@ private:
 
     template<typename T>
     std::vector<mtx::events::StateEvent<T>>
-    getStateEventsWithType(lmdb::txn &txn, const std::string &room_id)
+    getStateEventsWithType(lmdb::txn &txn,
+                           const std::string &room_id,
+                           mtx::events::EventType type = mtx::events::state_content_to_type<T>)
 
     {
-        constexpr auto type = mtx::events::state_content_to_type<T>;
-        static_assert(type != mtx::events::EventType::Unsupported,
-                      "Not a supported type in state events.");
-
         if (room_id.empty())
             return {};
 
@@ -638,8 +640,6 @@ private:
         return lmdb::dbi::open(txn, std::string(room_id + "/mentions").c_str(), MDB_CREATE);
     }
 
-    lmdb::dbi getPresenceDb(lmdb::txn &txn) { return lmdb::dbi::open(txn, "presence", MDB_CREATE); }
-
     lmdb::dbi getUserKeysDb(lmdb::txn &txn) { return lmdb::dbi::open(txn, "user_key", MDB_CREATE); }
 
     lmdb::dbi getVerificationDb(lmdb::txn &txn)
@@ -678,6 +678,7 @@ private:
     lmdb::dbi invitesDb_;
     lmdb::dbi readReceiptsDb_;
     lmdb::dbi notificationsDb_;
+    lmdb::dbi presenceDb_;
 
     lmdb::dbi devicesDb_;
     lmdb::dbi deviceKeysDb_;
