@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2017 Konstantinos Sideris <siderisk@auth.gr>
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
+// SPDX-FileCopyrightText: 2022 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -21,7 +22,6 @@
 #include "Utils.h"
 #include "encryption/DeviceVerificationFlow.h"
 #include "encryption/Olm.h"
-#include "ui/OverlayModal.h"
 #include "ui/Theme.h"
 #include "ui/UserProfile.h"
 #include "voip/CallManager.h"
@@ -43,14 +43,14 @@ Q_DECLARE_METATYPE(mtx::presence::PresenceState)
 Q_DECLARE_METATYPE(mtx::secret_storage::AesHmacSha2KeyDescription)
 Q_DECLARE_METATYPE(SecretsToDecrypt)
 
-ChatPage::ChatPage(QSharedPointer<UserSettings> userSettings, QWidget *parent)
-  : QWidget(parent)
+ChatPage::ChatPage(QSharedPointer<UserSettings> userSettings, QObject *parent)
+  : QObject(parent)
   , isConnected_(true)
   , userSettings_{userSettings}
   , notificationsManager(this)
   , callManager_(new CallManager(this))
 {
-    setObjectName("chatPage");
+    setObjectName(QStringLiteral("chatPage"));
 
     instance_ = this;
 
@@ -60,14 +60,8 @@ ChatPage::ChatPage(QSharedPointer<UserSettings> userSettings, QWidget *parent)
     qRegisterMetaType<mtx::secret_storage::AesHmacSha2KeyDescription>();
     qRegisterMetaType<SecretsToDecrypt>();
 
-    topLayout_ = new QHBoxLayout(this);
-    topLayout_->setSpacing(0);
-    topLayout_->setMargin(0);
-
     view_manager_ = new TimelineViewManager(callManager_, this);
 
-    topLayout_->addWidget(view_manager_->getWidget());
-
     connect(this,
             &ChatPage::downloadedSecrets,
             this,
@@ -153,7 +147,7 @@ ChatPage::ChatPage(QSharedPointer<UserSettings> userSettings, QWidget *parent)
             [this](const QString &roomid, const QString &eventid) {
                 Q_UNUSED(eventid)
                 view_manager_->rooms()->setCurrentRoom(roomid);
-                activateWindow();
+                MainWindow::instance()->requestActivate();
             });
     connect(&notificationsManager,
             &NotificationsManager::sendNotificationReply,
@@ -161,17 +155,9 @@ ChatPage::ChatPage(QSharedPointer<UserSettings> userSettings, QWidget *parent)
             [this](const QString &roomid, const QString &eventid, const QString &body) {
                 view_manager_->rooms()->setCurrentRoom(roomid);
                 view_manager_->queueReply(roomid, eventid, body);
-                activateWindow();
+                MainWindow::instance()->requestActivate();
             });
 
-    connect(QCoreApplication::instance(), &QCoreApplication::aboutToQuit, this, [this]() {
-        // ensure the qml context is shutdown before we destroy all other singletons
-        // Otherwise Qml will try to access the room list or settings, after they have been
-        // destroyed
-        topLayout_->removeWidget(view_manager_->getWidget());
-        delete view_manager_->getWidget();
-    });
-
     connect(
       this,
       &ChatPage::initializeViews,
@@ -182,8 +168,6 @@ ChatPage::ChatPage(QSharedPointer<UserSettings> userSettings, QWidget *parent)
             &ChatPage::initializeEmptyViews,
             view_manager_,
             &TimelineViewManager::initializeRoomlist);
-    connect(
-      this, &ChatPage::chatFocusChanged, view_manager_, &TimelineViewManager::chatFocusChanged);
     connect(this, &ChatPage::syncUI, this, [this](const mtx::responses::Sync &sync) {
         view_manager_->sync(sync);
 
@@ -200,7 +184,7 @@ ChatPage::ChatPage(QSharedPointer<UserSettings> userSettings, QWidget *parent)
         // TODO: Replace this once we have proper pushrules support. This is a horrible hack
         if (prevNotificationCount < notificationCount) {
             if (userSettings_->hasAlertOnNotification())
-                QApplication::alert(this);
+                MainWindow::instance()->alert(0);
         }
         prevNotificationCount = notificationCount;
 
@@ -279,12 +263,12 @@ ChatPage::deleteConfigs()
 {
     auto settings = UserSettings::instance()->qsettings();
 
-    if (UserSettings::instance()->profile() != "") {
-        settings->beginGroup("profile");
+    if (UserSettings::instance()->profile() != QLatin1String("")) {
+        settings->beginGroup(QStringLiteral("profile"));
         settings->beginGroup(UserSettings::instance()->profile());
     }
-    settings->beginGroup("auth");
-    settings->remove("");
+    settings->beginGroup(QStringLiteral("auth"));
+    settings->remove(QLatin1String(""));
     settings->endGroup(); // auth
 
     http::client()->shutdown();
@@ -330,7 +314,7 @@ ChatPage::bootstrap(QString userid, QString homeserver, QString token)
                     } else if (cacheVersion == cache::CacheVersion::Older) {
                         if (!cache::runMigrations()) {
                             QMessageBox::critical(
-                              this,
+                              nullptr,
                               tr("Cache migration failed!"),
                               tr("Migrating the cache to the current version failed. "
                                  "This can have different reasons. Please open an "
@@ -343,7 +327,7 @@ ChatPage::bootstrap(QString userid, QString homeserver, QString token)
                         return;
                     } else if (cacheVersion == cache::CacheVersion::Newer) {
                         QMessageBox::critical(
-                          this,
+                          nullptr,
                           tr("Incompatible cache version"),
                           tr("The cache on your disk is newer than this version of Nheko "
                              "supports. Please update Nheko or clear your cache."));
@@ -462,7 +446,7 @@ ChatPage::sendNotifications(const mtx::responses::Notifications &res)
             if (!cache::isNotificationSent(event_id)) {
                 const auto room_id = QString::fromStdString(item.room_id);
 
-                // We should only sent one notification per event.
+                // We should only send one notification per event.
                 cache::markSentNotification(event_id);
 
                 // Don't send a notification when the current room is opened.
@@ -689,7 +673,7 @@ ChatPage::joinRoomVia(const std::string &room_id,
     if (promptForConfirmation &&
         QMessageBox::Yes !=
           QMessageBox::question(
-            this,
+            nullptr,
             tr("Confirm join"),
             tr("Do you really want to join %1?").arg(QString::fromStdString(room_id))))
         return;
@@ -775,11 +759,10 @@ ChatPage::inviteUser(QString userid, QString reason)
 {
     auto room = currentRoom();
 
-    if (QMessageBox::question(this,
+    if (QMessageBox::question(nullptr,
                               tr("Confirm invite"),
                               tr("Do you really want to invite %1 (%2)?")
-                                .arg(cache::displayName(room, userid))
-                                .arg(userid)) != QMessageBox::Yes)
+                                .arg(cache::displayName(room, userid), userid)) != QMessageBox::Yes)
         return;
 
     http::client()->invite_user(
@@ -787,10 +770,11 @@ ChatPage::inviteUser(QString userid, QString reason)
       userid.toStdString(),
       [this, userid, room](const mtx::responses::Empty &, mtx::http::RequestErr err) {
           if (err) {
-              emit showNotification(tr("Failed to invite %1 to %2: %3")
-                                      .arg(userid)
-                                      .arg(room)
-                                      .arg(QString::fromStdString(err->matrix_error.error)));
+              nhlog::net()->error(
+                "Failed to invite {} to {}: {}", userid.toStdString(), room.toStdString(), *err);
+              emit showNotification(
+                tr("Failed to invite %1 to %2: %3")
+                  .arg(userid, room, QString::fromStdString(err->matrix_error.error)));
           } else
               emit showNotification(tr("Invited user: %1").arg(userid));
       },
@@ -801,22 +785,27 @@ ChatPage::kickUser(QString userid, QString reason)
 {
     auto room = currentRoom();
 
-    if (QMessageBox::question(this,
-                              tr("Confirm kick"),
-                              tr("Do you really want to kick %1 (%2)?")
-                                .arg(cache::displayName(room, userid))
-                                .arg(userid)) != QMessageBox::Yes)
+    bool confirmed;
+    reason =
+      QInputDialog::getText(nullptr,
+                            tr("Reason for the kick"),
+                            tr("Enter reason for kicking %1 (%2) or hit enter for no reason:")
+                              .arg(cache::displayName(room, userid), userid),
+                            QLineEdit::Normal,
+                            reason,
+                            &confirmed);
+    if (!confirmed) {
         return;
+    }
 
     http::client()->kick_user(
       room.toStdString(),
       userid.toStdString(),
       [this, userid, room](const mtx::responses::Empty &, mtx::http::RequestErr err) {
           if (err) {
-              emit showNotification(tr("Failed to kick %1 from %2: %3")
-                                      .arg(userid)
-                                      .arg(room)
-                                      .arg(QString::fromStdString(err->matrix_error.error)));
+              emit showNotification(
+                tr("Failed to kick %1 from %2: %3")
+                  .arg(userid, room, QString::fromStdString(err->matrix_error.error)));
           } else
               emit showNotification(tr("Kicked user: %1").arg(userid));
       },
@@ -827,22 +816,27 @@ ChatPage::banUser(QString userid, QString reason)
 {
     auto room = currentRoom();
 
-    if (QMessageBox::question(this,
-                              tr("Confirm ban"),
-                              tr("Do you really want to ban %1 (%2)?")
-                                .arg(cache::displayName(room, userid))
-                                .arg(userid)) != QMessageBox::Yes)
+    bool confirmed;
+    reason =
+      QInputDialog::getText(nullptr,
+                            tr("Reason for the ban"),
+                            tr("Enter reason for banning %1 (%2) or hit enter for no reason:")
+                              .arg(cache::displayName(room, userid), userid),
+                            QLineEdit::Normal,
+                            reason,
+                            &confirmed);
+    if (!confirmed) {
         return;
+    }
 
     http::client()->ban_user(
       room.toStdString(),
       userid.toStdString(),
       [this, userid, room](const mtx::responses::Empty &, mtx::http::RequestErr err) {
           if (err) {
-              emit showNotification(tr("Failed to ban %1 in %2: %3")
-                                      .arg(userid)
-                                      .arg(room)
-                                      .arg(QString::fromStdString(err->matrix_error.error)));
+              emit showNotification(
+                tr("Failed to ban %1 in %2: %3")
+                  .arg(userid, room, QString::fromStdString(err->matrix_error.error)));
           } else
               emit showNotification(tr("Banned user: %1").arg(userid));
       },
@@ -853,11 +847,10 @@ ChatPage::unbanUser(QString userid, QString reason)
 {
     auto room = currentRoom();
 
-    if (QMessageBox::question(this,
+    if (QMessageBox::question(nullptr,
                               tr("Confirm unban"),
                               tr("Do you really want to unban %1 (%2)?")
-                                .arg(cache::displayName(room, userid))
-                                .arg(userid)) != QMessageBox::Yes)
+                                .arg(cache::displayName(room, userid), userid)) != QMessageBox::Yes)
         return;
 
     http::client()->unban_user(
@@ -865,10 +858,9 @@ ChatPage::unbanUser(QString userid, QString reason)
       userid.toStdString(),
       [this, userid, room](const mtx::responses::Empty &, mtx::http::RequestErr err) {
           if (err) {
-              emit showNotification(tr("Failed to unban %1 in %2: %3")
-                                      .arg(userid)
-                                      .arg(room)
-                                      .arg(QString::fromStdString(err->matrix_error.error)));
+              emit showNotification(
+                tr("Failed to unban %1 in %2: %3")
+                  .arg(userid, room, QString::fromStdString(err->matrix_error.error)));
           } else
               emit showNotification(tr("Unbanned user: %1").arg(userid));
       },
@@ -884,7 +876,7 @@ ChatPage::receivedSessionKey(const std::string &room_id, const std::string &sess
 QString
 ChatPage::status() const
 {
-    return QString::fromStdString(cache::statusMessage(utils::localUser().toStdString()));
+    return QString::fromStdString(cache::presence(utils::localUser().toStdString()).status_msg);
 }
 
 void
@@ -951,7 +943,7 @@ ChatPage::ensureOneTimeKeyCount(const std::map<std::string, uint16_t> &counts)
           "Updated server key count {} {}", count->second, mtx::crypto::SIGNED_CURVE25519);
 
         if (count->second < MAX_ONETIME_KEYS) {
-            const int nkeys = MAX_ONETIME_KEYS - count->second;
+            const size_t nkeys = MAX_ONETIME_KEYS - count->second;
 
             nhlog::crypto()->info("uploading {} {} keys", nkeys, mtx::crypto::SIGNED_CURVE25519);
             olm::client()->generate_one_time_keys(nkeys);
@@ -1070,8 +1062,6 @@ ChatPage::initiateLogout()
 
         emit loggedOut();
     });
-
-    emit showOverlayProgressBar();
 }
 
 template<typename T>
@@ -1089,7 +1079,7 @@ ChatPage::decryptDownloadedSecrets(mtx::secret_storage::AesHmacSha2KeyDescriptio
                                    const SecretsToDecrypt &secrets)
 {
     QString text = QInputDialog::getText(
-      ChatPage::instance(),
+      nullptr,
       QCoreApplication::translate("CrossSigningSecrets", "Decrypt secrets"),
       keyDesc.name.empty()
         ? QCoreApplication::translate(
@@ -1121,7 +1111,7 @@ ChatPage::decryptDownloadedSecrets(mtx::secret_storage::AesHmacSha2KeyDescriptio
 
     if (!decryptionKey) {
         QMessageBox::information(
-          ChatPage::instance(),
+          nullptr,
           QCoreApplication::translate("CrossSigningSecrets", "Decryption failed"),
           QCoreApplication::translate("CrossSigningSecrets",
                                       "Failed to decrypt secrets with the "
@@ -1202,7 +1192,7 @@ ChatPage::startChat(QString userid)
     auto joined_rooms = cache::joinedRooms();
     auto room_infos   = cache::getRoomInfo(joined_rooms);
 
-    for (std::string room_id : joined_rooms) {
+    for (const std::string &room_id : joined_rooms) {
         if (room_infos[QString::fromStdString(room_id)].member_count == 2) {
             auto room_members = cache::roomMembers(room_id);
             if (std::find(room_members.begin(), room_members.end(), (userid).toStdString()) !=
@@ -1215,7 +1205,7 @@ ChatPage::startChat(QString userid)
 
     if (QMessageBox::Yes !=
         QMessageBox::question(
-          this,
+          nullptr,
           tr("Confirm invite"),
           tr("Do you really want to start a private chat with %1?").arg(userid)))
         return;
@@ -1231,23 +1221,23 @@ ChatPage::startChat(QString userid)
 }
 
 static QString
-mxidFromSegments(QStringRef sigil, QStringRef mxid)
+mxidFromSegments(QStringView sigil, QStringView mxid)
 {
     if (mxid.isEmpty())
-        return "";
+        return QString();
 
     auto mxid_ = QUrl::fromPercentEncoding(mxid.toUtf8());
 
-    if (sigil == "u") {
+    if (sigil == u"u") {
         return "@" + mxid_;
-    } else if (sigil == "roomid") {
+    } else if (sigil == u"roomid") {
         return "!" + mxid_;
-    } else if (sigil == "r") {
+    } else if (sigil == u"r") {
         return "#" + mxid_;
         //} else if (sigil == "group") {
         //        return "+" + mxid_;
     } else {
-        return "";
+        return QString();
     }
 }
 
@@ -1258,33 +1248,33 @@ ChatPage::handleMatrixUri(QString uri)
     QUrl uri_{uri};
 
     // Convert matrix.to URIs to proper format
-    if (uri_.scheme() == "https" && uri_.host() == "matrix.to") {
+    if (uri_.scheme() == QLatin1String("https") && uri_.host() == QLatin1String("matrix.to")) {
         QString p = uri_.fragment(QUrl::FullyEncoded);
-        if (p.startsWith("/"))
+        if (p.startsWith(QLatin1String("/")))
             p.remove(0, 1);
 
-        auto temp = p.split("?");
+        auto temp = p.split(QStringLiteral("?"));
         QString query;
         if (temp.size() >= 2)
             query = QUrl::fromPercentEncoding(temp.takeAt(1).toUtf8());
 
-        temp            = temp.first().split("/");
+        temp            = temp.first().split(QStringLiteral("/"));
         auto identifier = QUrl::fromPercentEncoding(temp.takeFirst().toUtf8());
         QString eventId = QUrl::fromPercentEncoding(temp.join('/').toUtf8());
         if (!identifier.isEmpty()) {
-            if (identifier.startsWith("@")) {
+            if (identifier.startsWith(QLatin1String("@"))) {
                 QByteArray newUri = "matrix:u/" + QUrl::toPercentEncoding(identifier.remove(0, 1));
                 if (!query.isEmpty())
                     newUri.append("?" + query.toUtf8());
                 return handleMatrixUri(QUrl::fromEncoded(newUri));
-            } else if (identifier.startsWith("#")) {
+            } else if (identifier.startsWith(QLatin1String("#"))) {
                 QByteArray newUri = "matrix:r/" + QUrl::toPercentEncoding(identifier.remove(0, 1));
                 if (!eventId.isEmpty())
                     newUri.append("/e/" + QUrl::toPercentEncoding(eventId.remove(0, 1)));
                 if (!query.isEmpty())
                     newUri.append("?" + query.toUtf8());
                 return handleMatrixUri(QUrl::fromEncoded(newUri));
-            } else if (identifier.startsWith("!")) {
+            } else if (identifier.startsWith(QLatin1String("!"))) {
                 QByteArray newUri =
                   "matrix:roomid/" + QUrl::toPercentEncoding(identifier.remove(0, 1));
                 if (!eventId.isEmpty())
@@ -1297,13 +1287,13 @@ ChatPage::handleMatrixUri(QString uri)
     }
 
     // non-matrix URIs are not handled by us, return false
-    if (uri_.scheme() != "matrix")
+    if (uri_.scheme() != QLatin1String("matrix"))
         return false;
 
     auto tempPath = uri_.path(QUrl::ComponentFormattingOption::FullyEncoded);
     if (tempPath.startsWith('/'))
         tempPath.remove(0, 1);
-    auto segments = tempPath.splitRef('/');
+    auto segments = QStringView(tempPath).split('/');
 
     if (segments.size() != 2 && segments.size() != 4)
         return false;
@@ -1314,7 +1304,7 @@ ChatPage::handleMatrixUri(QString uri)
         return false;
 
     QString mxid2;
-    if (segments.size() == 4 && segments[2] == "e") {
+    if (segments.size() == 4 && segments[2] == QStringView(u"e")) {
         if (segments[3].isEmpty())
             return false;
         else
@@ -1324,18 +1314,20 @@ ChatPage::handleMatrixUri(QString uri)
     std::vector<std::string> vias;
     QString action;
 
-    for (QString item :
-         uri_.query(QUrl::ComponentFormattingOption::FullyEncoded).split('&', Qt::SkipEmptyParts)) {
+    auto items =
+      uri_.query(QUrl::ComponentFormattingOption::FullyEncoded).split('&', Qt::SkipEmptyParts);
+    for (QString item : qAsConst(items)) {
         nhlog::ui()->info("item: {}", item.toStdString());
 
-        if (item.startsWith("action=")) {
-            action = item.remove("action=");
-        } else if (item.startsWith("via=")) {
-            vias.push_back(QUrl::fromPercentEncoding(item.remove("via=").toUtf8()).toStdString());
+        if (item.startsWith(QLatin1String("action="))) {
+            action = item.remove(QStringLiteral("action="));
+        } else if (item.startsWith(QLatin1String("via="))) {
+            vias.push_back(QUrl::fromPercentEncoding(item.remove(QStringLiteral("via=")).toUtf8())
+                             .toStdString());
         }
     }
 
-    if (sigil1 == "u") {
+    if (sigil1 == u"u") {
         if (action.isEmpty()) {
             auto t = view_manager_->rooms()->currentRoom();
             if (t && cache::isRoomMember(mxid1.toStdString(), t->roomId().toStdString())) {
@@ -1343,15 +1335,15 @@ ChatPage::handleMatrixUri(QString uri)
                 return true;
             }
             emit view_manager_->openGlobalUserProfile(mxid1);
-        } else if (action == "chat") {
+        } else if (action == u"chat") {
             this->startChat(mxid1);
         }
         return true;
-    } else if (sigil1 == "roomid") {
+    } else if (sigil1 == u"roomid") {
         auto joined_rooms = cache::joinedRooms();
         auto targetRoomId = mxid1.toStdString();
 
-        for (auto roomid : joined_rooms) {
+        for (const auto &roomid : joined_rooms) {
             if (roomid == targetRoomId) {
                 view_manager_->rooms()->setCurrentRoom(mxid1);
                 if (!mxid2.isEmpty())
@@ -1360,16 +1352,16 @@ ChatPage::handleMatrixUri(QString uri)
             }
         }
 
-        if (action == "join" || action.isEmpty()) {
+        if (action == u"join" || action.isEmpty()) {
             joinRoomVia(targetRoomId, vias);
             return true;
         }
         return false;
-    } else if (sigil1 == "r") {
+    } else if (sigil1 == u"r") {
         auto joined_rooms    = cache::joinedRooms();
         auto targetRoomAlias = mxid1.toStdString();
 
-        for (auto roomid : joined_rooms) {
+        for (const auto &roomid : joined_rooms) {
             auto aliases = cache::client()->getRoomAliases(roomid);
             if (aliases) {
                 if (aliases->alias == targetRoomAlias) {
@@ -1381,7 +1373,7 @@ ChatPage::handleMatrixUri(QString uri)
             }
         }
 
-        if (action == "join" || action.isEmpty()) {
+        if (action == u"join" || action.isEmpty()) {
             joinRoomVia(mxid1.toStdString(), vias);
             return true;
         }
@@ -1399,7 +1391,7 @@ ChatPage::handleMatrixUri(const QUrl &uri)
 bool
 ChatPage::isRoomActive(const QString &room_id)
 {
-    return isActiveWindow() && currentRoom() == room_id;
+    return MainWindow::instance()->isActive() && currentRoom() == room_id;
 }
 
 QString
@@ -1408,5 +1400,5 @@ ChatPage::currentRoom() const
     if (view_manager_->rooms()->currentRoom())
         return view_manager_->rooms()->currentRoom()->roomId();
     else
-        return "";
+        return QString();
 }
