@@ -1,4 +1,5 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
+// SPDX-FileCopyrightText: 2022 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -27,23 +28,30 @@ MxcImageProvider::requestImageResponse(const QString &id, const QSize &requested
     auto id_      = id;
     bool crop     = true;
     double radius = 0;
+    auto size     = requestedSize;
+
+    if (requestedSize.width() == 0 && requestedSize.height() == 0)
+        size = QSize();
 
     auto queryStart = id.lastIndexOf('?');
     if (queryStart != -1) {
         id_            = id.left(queryStart);
-        auto query     = id.midRef(queryStart + 1);
+        auto query     = QStringView(id).mid(queryStart + 1);
         auto queryBits = query.split('&');
 
         for (auto b : queryBits) {
-            if (b == "scale") {
+            if (b == QStringView(u"scale")) {
                 crop = false;
-            } else if (b.startsWith("radius=")) {
+            } else if (b.startsWith(QStringView(u"radius="))) {
                 radius = b.mid(7).toDouble();
+            } else if (b.startsWith(u"height=")) {
+                size.setHeight(b.mid(7).toInt());
+                size.setWidth(0);
             }
         }
     }
 
-    return new MxcImageResponse(id_, crop, radius, requestedSize);
+    return new MxcImageResponse(id_, crop, radius, size);
 }
 
 void
@@ -57,9 +65,9 @@ MxcImageRunnable::run()
     MxcImageProvider::download(
       m_id,
       m_requestedSize,
-      [this](QString, QSize, QImage image, QString) {
+      [this](QString id, QSize, QImage image, QString) {
           if (image.isNull()) {
-              emit error("Failed to download image.");
+              emit error(QStringLiteral("Failed to download image: %1").arg(id));
           } else {
               emit done(image);
           }
@@ -105,7 +113,7 @@ MxcImageProvider::download(const QString &id,
         // Protect against synapse not following the spec:
         // https://github.com/matrix-org/synapse/issues/5302
         && requestedSize.height() <= 600 && requestedSize.width() <= 800) {
-        QString fileName = QString("%1_%2x%3_%4_radius%5")
+        QString fileName = QStringLiteral("%1_%2x%3_%4_radius%5")
                              .arg(QString::fromUtf8(id.toUtf8().toBase64(
                                QByteArray::Base64UrlEncoding | QByteArray::OmitTrailingEquals)))
                              .arg(requestedSize.width())
@@ -120,7 +128,12 @@ MxcImageProvider::download(const QString &id,
         if (fileInfo.exists()) {
             QImage image = utils::readImageFromFile(fileInfo.absoluteFilePath());
             if (!image.isNull()) {
-                image = image.scaled(requestedSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
+                if (requestedSize.width() <= 0) {
+                    image = image.scaledToHeight(requestedSize.height(), Qt::SmoothTransformation);
+                } else {
+                    image =
+                      image.scaled(requestedSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
+                }
 
                 if (radius != 0) {
                     image = clipRadius(std::move(image), radius);
@@ -144,21 +157,25 @@ MxcImageProvider::download(const QString &id,
                                                             mtx::http::RequestErr err) {
               if (err || res.empty()) {
                   download(id, QSize(), then, crop, radius);
-
                   return;
               }
 
               auto data    = QByteArray(res.data(), (int)res.size());
               QImage image = utils::readImage(data);
               if (!image.isNull()) {
-                  image =
-                    image.scaled(requestedSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
+                  if (requestedSize.width() <= 0) {
+                      image =
+                        image.scaledToHeight(requestedSize.height(), Qt::SmoothTransformation);
+                  } else {
+                      image =
+                        image.scaled(requestedSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
+                  }
 
                   if (radius != 0) {
                       image = clipRadius(std::move(image), radius);
                   }
               }
-              image.setText("mxc url", "mxc://" + id);
+              image.setText(QStringLiteral("mxc url"), "mxc://" + id);
               if (image.save(fileInfo.absoluteFilePath(), "png"))
                   nhlog::ui()->debug("Wrote: {}", fileInfo.absoluteFilePath().toStdString());
               else
@@ -169,7 +186,7 @@ MxcImageProvider::download(const QString &id,
           });
     } else {
         try {
-            QString fileName = QString("%1_radius%2")
+            QString fileName = QStringLiteral("%1_radius%2")
                                  .arg(QString::fromUtf8(id.toUtf8().toBase64(
                                    QByteArray::Base64UrlEncoding | QByteArray::OmitTrailingEquals)))
                                  .arg(radius);
@@ -189,7 +206,7 @@ MxcImageProvider::download(const QString &id,
                       mtx::crypto::decrypt_file(fileData.toStdString(), encryptionInfo.value()));
                     auto data    = QByteArray(tempData.data(), (int)tempData.size());
                     QImage image = utils::readImage(data);
-                    image.setText("mxc url", "mxc://" + id);
+                    image.setText(QStringLiteral("mxc url"), "mxc://" + id);
                     if (!image.isNull()) {
                         if (radius != 0) {
                             image = clipRadius(std::move(image), radius);
@@ -219,14 +236,17 @@ MxcImageProvider::download(const QString &id,
                 const std::string &originalFilename,
                 mtx::http::RequestErr err) {
                   if (err) {
-                      then(id, QSize(), {}, "");
+                      nhlog::net()->error("Failed to download {}: {}", id.toStdString(), *err);
+                      then(id, QSize(), {}, QLatin1String(""));
                       return;
                   }
 
                   auto tempData = res;
                   QFile f(fileInfo.absoluteFilePath());
                   if (!f.open(QIODevice::Truncate | QIODevice::WriteOnly)) {
-                      then(id, QSize(), {}, "");
+                      nhlog::net()->error(
+                        "Failed to write {}: {}", id.toStdString(), f.errorString().toStdString());
+                      then(id, QSize(), {}, QLatin1String(""));
                       return;
                   }
                   f.write(tempData.data(), tempData.size());
@@ -241,8 +261,9 @@ MxcImageProvider::download(const QString &id,
                           image = clipRadius(std::move(image), radius);
                       }
 
-                      image.setText("original filename", QString::fromStdString(originalFilename));
-                      image.setText("mxc url", "mxc://" + id);
+                      image.setText(QStringLiteral("original filename"),
+                                    QString::fromStdString(originalFilename));
+                      image.setText(QStringLiteral("mxc url"), "mxc://" + id);
                       then(id, requestedSize, image, fileInfo.absoluteFilePath());
                       return;
                   }
@@ -252,8 +273,9 @@ MxcImageProvider::download(const QString &id,
                       image = clipRadius(std::move(image), radius);
                   }
 
-                  image.setText("original filename", QString::fromStdString(originalFilename));
-                  image.setText("mxc url", "mxc://" + id);
+                  image.setText(QStringLiteral("original filename"),
+                                QString::fromStdString(originalFilename));
+                  image.setText(QStringLiteral("mxc url"), "mxc://" + id);
                   then(id, requestedSize, image, fileInfo.absoluteFilePath());
               });
         } catch (std::exception &e) {
