@@ -1,16 +1,15 @@
 // SPDX-FileCopyrightText: 2017 Konstantinos Sideris <siderisk@auth.gr>
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
+// SPDX-FileCopyrightText: 2022 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #pragma once
 
-#include <QFontDatabase>
-#include <QFrame>
+#include <QAbstractListModel>
 #include <QProcessEnvironment>
 #include <QSettings>
 #include <QSharedPointer>
-#include <QWidget>
 
 #include "JdenticonProvider.h"
 #include <optional>
@@ -41,6 +40,8 @@ class UserSettings : public QObject
     Q_PROPERTY(bool startInTray READ startInTray WRITE setStartInTray NOTIFY startInTrayChanged)
     Q_PROPERTY(bool groupView READ groupView WRITE setGroupView NOTIFY groupViewStateChanged)
     Q_PROPERTY(bool markdown READ markdown WRITE setMarkdown NOTIFY markdownChanged)
+    Q_PROPERTY(bool bubbles READ bubbles WRITE setBubbles NOTIFY bubblesChanged)
+    Q_PROPERTY(bool smallAvatars READ smallAvatars WRITE setSmallAvatars NOTIFY smallAvatarsChanged)
     Q_PROPERTY(bool animateImagesOnHover READ animateImagesOnHover WRITE setAnimateImagesOnHover
                  NOTIFY animateImagesOnHoverChanged)
     Q_PROPERTY(bool typingNotifications READ typingNotifications WRITE setTypingNotifications NOTIFY
@@ -104,10 +105,16 @@ class UserSettings : public QObject
     Q_PROPERTY(bool disableCertificateValidation READ disableCertificateValidation WRITE
                  setDisableCertificateValidation NOTIFY disableCertificateValidationChanged)
     Q_PROPERTY(bool useIdenticon READ useIdenticon WRITE setUseIdenticon NOTIFY useIdenticonChanged)
+    Q_PROPERTY(bool openImageExternal READ openImageExternal WRITE setOpenImageExternal NOTIFY
+                 openImageExternalChanged)
+    Q_PROPERTY(bool openVideoExternal READ openVideoExternal WRITE setOpenVideoExternal NOTIFY
+                 openVideoExternalChanged)
 
     Q_PROPERTY(QStringList hiddenPins READ hiddenPins WRITE setHiddenPins NOTIFY hiddenPinsChanged)
     Q_PROPERTY(QStringList recentReactions READ recentReactions WRITE setRecentReactions NOTIFY
                  recentReactionsChanged)
+    Q_PROPERTY(QStringList hiddenWidgets READ hiddenWidgets WRITE setHiddenWidgets NOTIFY
+                 hiddenWidgetsChanged)
 
     UserSettings();
 
@@ -140,6 +147,8 @@ public:
     void setEmojiFontFamily(QString family);
     void setGroupView(bool state);
     void setMarkdown(bool state);
+    void setBubbles(bool state);
+    void setSmallAvatars(bool state);
     void setAnimateImagesOnHover(bool state);
     void setReadReceipts(bool state);
     void setTypingNotifications(bool state);
@@ -176,8 +185,11 @@ public:
     void setDisableCertificateValidation(bool disabled);
     void setHiddenTags(QStringList hiddenTags);
     void setHiddenPins(QStringList hiddenTags);
+    void setHiddenWidgets(QStringList hiddenTags);
     void setRecentReactions(QStringList recent);
     void setUseIdenticon(bool state);
+    void setOpenImageExternal(bool state);
+    void setOpenVideoExternal(bool state);
     void setCollapsedSpaces(QList<QStringList> spaces);
 
     QString theme() const { return !theme_.isEmpty() ? theme_ : defaultTheme_; }
@@ -191,6 +203,8 @@ public:
     bool privacyScreen() const { return privacyScreen_; }
     int privacyScreenTimeout() const { return privacyScreenTimeout_; }
     bool markdown() const { return markdown_; }
+    bool bubbles() const { return bubbles_; }
+    bool smallAvatars() const { return smallAvatars_; }
     bool animateImagesOnHover() const { return animateImagesOnHover_; }
     bool typingNotifications() const { return typingNotifications_; }
     bool sortByImportance() const { return sortByImportance_; }
@@ -207,7 +221,7 @@ public:
     QString font() const { return font_; }
     QString emojiFont() const
     {
-        if (emojiFont_ == "Default") {
+        if (emojiFont_ == QLatin1String("Default")) {
             return tr("Default");
         }
 
@@ -235,8 +249,11 @@ public:
     bool disableCertificateValidation() const { return disableCertificateValidation_; }
     QStringList hiddenTags() const { return hiddenTags_; }
     QStringList hiddenPins() const { return hiddenPins_; }
+    QStringList hiddenWidgets() const { return hiddenWidgets_; }
     QStringList recentReactions() const { return recentReactions_; }
     bool useIdenticon() const { return useIdenticon_ && JdenticonProvider::isAvailable(); }
+    bool openImageExternal() const { return openImageExternal_; }
+    bool openVideoExternal() const { return openVideoExternal_; }
     QList<QStringList> collapsedSpaces() const { return collapsedSpaces_; }
 
 signals:
@@ -248,6 +265,8 @@ signals:
     void trayChanged(bool state);
     void startInTrayChanged(bool state);
     void markdownChanged(bool state);
+    void bubblesChanged(bool state);
+    void smallAvatarsChanged(bool state);
     void animateImagesOnHoverChanged(bool state);
     void typingNotificationsChanged(bool state);
     void buttonInTimelineChanged(bool state);
@@ -286,15 +305,19 @@ signals:
     void homeserverChanged(QString homeserver);
     void disableCertificateValidationChanged(bool disabled);
     void useIdenticonChanged(bool state);
+    void openImageExternalChanged(bool state);
+    void openVideoExternalChanged(bool state);
     void hiddenPinsChanged();
+    void hiddenWidgetsChanged();
     void recentReactionsChanged();
 
 private:
     // Default to system theme if QT_QPA_PLATFORMTHEME var is set.
-    QString defaultTheme_ =
-      QProcessEnvironment::systemEnvironment().value("QT_QPA_PLATFORMTHEME", "").isEmpty()
-        ? "light"
-        : "system";
+    QString defaultTheme_ = QProcessEnvironment::systemEnvironment()
+                                .value(QStringLiteral("QT_QPA_PLATFORMTHEME"), QLatin1String(""))
+                                .isEmpty()
+                              ? "light"
+                              : "system";
     QString theme_;
     bool messageHoverHighlight_;
     bool enlargeEmojiOnlyMessages_;
@@ -302,6 +325,8 @@ private:
     bool startInTray_;
     bool groupView_;
     bool markdown_;
+    bool bubbles_;
+    bool smallAvatars_;
     bool animateImagesOnHover_;
     bool typingNotifications_;
     bool sortByImportance_;
@@ -342,98 +367,141 @@ private:
     QString homeserver_;
     QStringList hiddenTags_;
     QStringList hiddenPins_;
+    QStringList hiddenWidgets_;
     QStringList recentReactions_;
     QList<QStringList> collapsedSpaces_;
     bool useIdenticon_;
+    bool openImageExternal_;
+    bool openVideoExternal_;
 
     QSettings settings;
 
     static QSharedPointer<UserSettings> instance_;
 };
 
-class HorizontalLine : public QFrame
+class UserSettingsModel : public QAbstractListModel
 {
     Q_OBJECT
 
-public:
-    HorizontalLine(QWidget *parent = nullptr);
-};
+    enum Indices
+    {
+        GeneralSection,
+        Theme,
+        MobileMode,
+#ifndef Q_OS_MAC
+        ScaleFactor,
+#endif
+        Font,
+        FontSize,
+        EmojiFont,
+        AvatarCircles,
+        UseIdenticon,
+        PrivacyScreen,
+        PrivacyScreenTimeout,
 
-class UserSettingsPage : public QWidget
-{
-    Q_OBJECT
+        TimelineSection,
+        TimelineMaxWidth,
+        MessageHoverHighlight,
+        EnlargeEmojiOnlyMessages,
+        AnimateImagesOnHover,
+        OpenImageExternal,
+        OpenVideoExternal,
+        ButtonsInTimeline,
+        TypingNotifications,
+        ReadReceipts,
+        Markdown,
+        Bubbles,
+        SmallAvatars,
+        SidebarSection,
+        GroupView,
+        SortByImportance,
+        DecryptSidebar,
+
+        TraySection,
+        Tray,
+        StartInTray,
+
+        NotificationsSection,
+        DesktopNotifications,
+        AlertOnNotification,
+
+        VoipSection,
+        UseStunServer,
+        Microphone,
+        Camera,
+        CameraResolution,
+        CameraFrameRate,
+        Ringtone,
+
+        EncryptionSection,
+        OnlyShareKeysWithVerifiedUsers,
+        ShareKeysWithTrustedUsers,
+        SessionKeys,
+        UseOnlineKeyBackup,
+        OnlineBackupKey,
+        SelfSigningKey,
+        UserSigningKey,
+        MasterKey,
+        CrossSigningSecrets,
+        DeviceId,
+        DeviceFingerprint,
+
+        LoginInfoSection,
+        UserId,
+        Homeserver,
+        Profile,
+        Version,
+        Platform,
+        COUNT,
+        // hidden for now
+        AccessToken,
+#ifdef Q_OS_MAC
+        ScaleFactor,
+#endif
+    };
 
 public:
-    UserSettingsPage(QSharedPointer<UserSettings> settings, QWidget *parent = nullptr);
+    enum Types
+    {
+        Toggle,
+        ReadOnlyText,
+        Options,
+        Integer,
+        Double,
+        SectionTitle,
+        SectionBar,
+        KeyStatus,
+        SessionKeyImportExport,
+        XSignKeysRequestDownload,
+    };
+    Q_ENUM(Types);
 
-protected:
-    void showEvent(QShowEvent *event) override;
-    void paintEvent(QPaintEvent *event) override;
+    enum Roles
+    {
+        Name,
+        Description,
+        Value,
+        Type,
+        ValueLowerBound,
+        ValueUpperBound,
+        ValueStep,
+        Values,
+        Good,
+        Enabled,
+    };
 
-signals:
-    void moveBack();
-    void trayOptionChanged(bool value);
-    void themeChanged();
-    void decryptSidebarChanged();
+    UserSettingsModel(QObject *parent = nullptr);
+    QHash<int, QByteArray> roleNames() const override;
+    int rowCount(const QModelIndex &parent = QModelIndex()) const override
+    {
+        (void)parent;
+        return (int)COUNT;
+    }
+    QVariant data(const QModelIndex &index, int role) const override;
+    bool setData(const QModelIndex &index, const QVariant &value, int role) override;
 
-public slots:
-    void updateSecretStatus();
-
-private slots:
-    void importSessionKeys();
-    void exportSessionKeys();
-
-private:
-    // Layouts
-    QVBoxLayout *topLayout_;
-    QHBoxLayout *topBarLayout_;
-    QFormLayout *formLayout_;
-
-    // Shared settings object.
-    QSharedPointer<UserSettings> settings_;
-
-    Toggle *trayToggle_;
-    Toggle *startInTrayToggle_;
-    Toggle *groupViewToggle_;
-    Toggle *timelineButtonsToggle_;
-    Toggle *typingNotifications_;
-    Toggle *messageHoverHighlight_;
-    Toggle *enlargeEmojiOnlyMessages_;
-    Toggle *sortByImportance_;
-    Toggle *readReceipts_;
-    Toggle *markdown_;
-    Toggle *animateImagesOnHover_;
-    Toggle *desktopNotifications_;
-    Toggle *alertOnNotification_;
-    Toggle *avatarCircles_;
-    Toggle *useIdenticon_;
-    Toggle *useStunServer_;
-    Toggle *decryptSidebar_;
-    Toggle *privacyScreen_;
-    QSpinBox *privacyScreenTimeout_;
-    Toggle *shareKeysWithTrustedUsers_;
-    Toggle *onlyShareKeysWithVerifiedUsers_;
-    Toggle *useOnlineKeyBackup_;
-    Toggle *mobileMode_;
-    QLabel *deviceFingerprintValue_;
-    QLabel *deviceIdValue_;
-    QLabel *backupSecretCached;
-    QLabel *masterSecretCached;
-    QLabel *selfSigningSecretCached;
-    QLabel *userSigningSecretCached;
-
-    QComboBox *themeCombo_;
-    QComboBox *scaleFactorCombo_;
-    QComboBox *fontSizeCombo_;
-    QFontComboBox *fontSelectionCombo_;
-    QComboBox *emojiFontSelectionCombo_;
-    QComboBox *ringtoneCombo_;
-    QComboBox *microphoneCombo_;
-    QComboBox *cameraCombo_;
-    QComboBox *cameraResolutionCombo_;
-    QComboBox *cameraFrameRateCombo_;
-
-    QSpinBox *timelineMaxWidthSpin_;
-
-    int sideMargin_ = 0;
+    Q_INVOKABLE void importSessionKeys();
+    Q_INVOKABLE void exportSessionKeys();
+    Q_INVOKABLE void requestCrossSigningSecrets();
+    Q_INVOKABLE void downloadCrossSigningSecrets();
 };
