@@ -1,4 +1,5 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
+// SPDX-FileCopyrightText: 2022 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -8,7 +9,6 @@
 #include <QBuffer>
 #include <QComboBox>
 #include <QCryptographicHash>
-#include <QDesktopWidget>
 #include <QGuiApplication>
 #include <QImageReader>
 #include <QProcessEnvironment>
@@ -17,6 +17,7 @@
 #include <QStringBuilder>
 #include <QTextBoundaryFinder>
 #include <QTextDocument>
+#include <QWindow>
 #include <QXmlStreamReader>
 
 #include <array>
@@ -59,7 +60,7 @@ std::string
 utils::stripReplyFromBody(const std::string &bodyi)
 {
     QString body = QString::fromStdString(bodyi);
-    if (body.startsWith("> <")) {
+    if (body.startsWith(QLatin1String("> <"))) {
         auto segments = body.split('\n');
         while (!segments.isEmpty() && segments.begin()->startsWith('>'))
             segments.erase(segments.begin());
@@ -68,7 +69,7 @@ utils::stripReplyFromBody(const std::string &bodyi)
         body = segments.join('\n');
     }
 
-    body.replace("@room", QString::fromUtf8("@\u2060room"));
+    body.replace(QLatin1String("@room"), QString::fromUtf8("@\u2060room"));
     return body.toStdString();
 }
 
@@ -76,9 +77,9 @@ std::string
 utils::stripReplyFromFormattedBody(const std::string &formatted_bodyi)
 {
     QString formatted_body = QString::fromStdString(formatted_bodyi);
-    formatted_body.remove(QRegularExpression("<mx-reply>.*</mx-reply>",
+    formatted_body.remove(QRegularExpression(QStringLiteral("<mx-reply>.*</mx-reply>"),
                                              QRegularExpression::DotMatchesEverythingOption));
-    formatted_body.replace("@room", QString::fromUtf8("@\u2060room"));
+    formatted_body.replace(QLatin1String("@room"), QString::fromUtf8("@\u2060room"));
     return formatted_body.toStdString();
 }
 
@@ -135,8 +136,13 @@ utils::replaceEmoji(const QString &body)
                 fmtBody += QStringLiteral("<font face=\"") % UserSettings::instance()->emojiFont() %
                            QStringLiteral("\">");
                 insideFontBlock = true;
+            } else if (code == 0xfe0f) {
+                // BUG(Nico):
+                // Workaround https://bugreports.qt.io/browse/QTBUG-97401
+                // See also https://github.com/matrix-org/matrix-react-sdk/pull/1458/files
+                // Nheko bug: https://github.com/Nheko-Reborn/nheko/issues/439
+                continue;
             }
-
         } else {
             if (insideFontBlock) {
                 fmtBody += QStringLiteral("</font>");
@@ -165,14 +171,14 @@ utils::setScaleFactor(float factor)
         return;
 
     QSettings settings;
-    settings.setValue("settings/scale_factor", factor);
+    settings.setValue(QStringLiteral("settings/scale_factor"), factor);
 }
 
 float
 utils::scaleFactor()
 {
     QSettings settings;
-    return settings.value("settings/scale_factor", -1).toFloat();
+    return settings.value(QStringLiteral("settings/scale_factor"), -1).toFloat();
 }
 
 QString
@@ -186,7 +192,7 @@ utils::descriptiveTime(const QDateTime &then)
     else if (days < 2)
         return QString(QCoreApplication::translate("descriptiveTime", "Yesterday"));
     else if (days < 7)
-        return then.toString("dddd");
+        return then.toString(QStringLiteral("dddd"));
 
     return QLocale::system().toString(then.date(), QLocale::ShortFormat);
 }
@@ -238,8 +244,8 @@ utils::getMessageDescription(const TimelineEvent &event,
 
         DescInfo info;
         info.userid = sender;
-        info.body =
-          QString(" %1").arg(messageDescription<Encrypted>(username, "", sender == localUser));
+        info.body   = QStringLiteral(" %1").arg(
+          messageDescription<Encrypted>(username, QLatin1String(""), sender == localUser));
         info.timestamp       = msg->origin_server_ts;
         info.descriptiveTime = utils::descriptiveTime(ts);
         info.event_id        = QString::fromStdString(msg->event_id);
@@ -257,8 +263,8 @@ utils::firstChar(const QString &input)
     if (input.isEmpty())
         return input;
 
-    for (auto const &c : input.toUcs4()) {
-        if (QString::fromUcs4(&c, 1) != QString("#"))
+    for (auto const &c : input.toStdU32String()) {
+        if (QString::fromUcs4(&c, 1) != QStringLiteral("#"))
             return QString::fromUcs4(&c, 1).toUpper();
     }
 
@@ -327,7 +333,7 @@ utils::event_body(const mtx::events::collections::TimelineEvents &e)
     if (auto ev = std::get_if<RoomEvent<msg::Video>>(&e); ev != nullptr)
         return QString::fromStdString(ev->content.body);
 
-    return "";
+    return QString();
 }
 
 QPixmap
@@ -372,11 +378,10 @@ utils::mxcToHttp(const QUrl &url, const QString &server, int port)
 {
     auto mxcParts = mtx::client::utils::parse_mxc_url(url.toString().toStdString());
 
-    return QString("https://%1:%2/_matrix/media/r0/download/%3/%4")
+    return QStringLiteral("https://%1:%2/_matrix/media/r0/download/%3/%4")
       .arg(server)
       .arg(port)
-      .arg(QString::fromStdString(mxcParts.server))
-      .arg(QString::fromStdString(mxcParts.media_id));
+      .arg(QString::fromStdString(mxcParts.server), QString::fromStdString(mxcParts.media_id));
 }
 
 QString
@@ -389,8 +394,8 @@ utils::humanReadableFingerprint(const QString &ed25519)
 {
     QString fingerprint;
     for (int i = 0; i < ed25519.length(); i = i + 4) {
-        fingerprint.append(ed25519.midRef(i, 4));
-        if (i > 0 && i % 16 == 12)
+        fingerprint.append(QStringView(ed25519).mid(i, 4));
+        if (i > 0 && i == 20)
             fingerprint.append('\n');
         else if (i < ed25519.length())
             fingerprint.append(' ');
@@ -404,8 +409,9 @@ utils::linkifyMessage(const QString &body)
     // Convert to valid XML.
     auto doc = body;
     doc.replace(conf::strings::url_regex, conf::strings::url_html);
-    doc.replace(QRegularExpression("\\b(?<![\"'])(?>(matrix:[\\S]{5,}))(?![\"'])\\b"),
-                conf::strings::url_html);
+    doc.replace(
+      QRegularExpression(QStringLiteral("\\b(?<![\"'])(?>(matrix:[\\S]{5,}))(?![\"'])\\b")),
+      conf::strings::url_html);
 
     return doc;
 }
@@ -513,10 +519,11 @@ utils::markdownToHtml(const QString &text, bool rainbowify)
             while ((boundaryEnd = tbf.toNextBoundary()) != -1) {
                 charIdx++;
                 // Split text to get current char
-                auto curChar  = nodeText.midRef(boundaryStart, boundaryEnd - boundaryStart);
+                auto curChar =
+                  QStringView(nodeText).mid(boundaryStart, boundaryEnd - boundaryStart);
                 boundaryStart = boundaryEnd;
                 // Don't rainbowify whitespaces
-                if (curChar.trimmed().isEmpty() || codepointIsEmoji(curChar.toUcs4().first())) {
+                if (curChar.trimmed().isEmpty() || codepointIsEmoji(curChar.toUcs4().at(0))) {
                     buf.append(curChar);
                     continue;
                 }
@@ -529,7 +536,7 @@ utils::markdownToHtml(const QString &text, bool rainbowify)
                 auto colorString = color.name(QColor::NameFormat::HexRgb);
                 // create HTML element for current char
                 auto curCharColored =
-                  QString("<font color=\"%0\">%1</font>").arg(colorString).arg(curChar);
+                  QStringLiteral("<font color=\"%0\">%1</font>").arg(colorString).arg(curChar);
                 // append colored HTML element to buffer
                 buf.append(curCharColored);
             }
@@ -556,7 +563,8 @@ utils::markdownToHtml(const QString &text, bool rainbowify)
 
     auto result = linkifyMessage(escapeBlacklistedHtml(QString::fromStdString(html))).trimmed();
 
-    if (result.count("<p>") == 1 && result.startsWith("<p>") && result.endsWith("</p>")) {
+    if (result.count(QStringLiteral("<p>")) == 1 && result.startsWith(QLatin1String("<p>")) &&
+        result.endsWith(QLatin1String("</p>"))) {
         result = result.mid(3, result.size() - 3 - 4);
     }
 
@@ -571,16 +579,16 @@ utils::getFormattedQuoteBody(const RelatedInfo &related, const QString &html)
 
         switch (related.type) {
         case MsgType::File: {
-            return "sent a file.";
+            return QStringLiteral("sent a file.");
         }
         case MsgType::Image: {
-            return "sent an image.";
+            return QStringLiteral("sent an image.");
         }
         case MsgType::Audio: {
-            return "sent an audio file.";
+            return QStringLiteral("sent an audio file.");
         }
         case MsgType::Video: {
-            return "sent a video";
+            return QStringLiteral("sent a video");
         }
         default: {
             return related.quoted_formatted_body;
@@ -606,16 +614,16 @@ utils::getQuoteBody(const RelatedInfo &related)
 
     switch (related.type) {
     case MsgType::File: {
-        return "sent a file.";
+        return QStringLiteral("sent a file.");
     }
     case MsgType::Image: {
-        return "sent an image.";
+        return QStringLiteral("sent an image.");
     }
     case MsgType::Audio: {
-        return "sent an audio file.";
+        return QStringLiteral("sent an audio file.");
     }
     case MsgType::Video: {
-        return "sent a video";
+        return QStringLiteral("sent a video");
     }
     default: {
         return related.quoted_body;
@@ -628,10 +636,10 @@ utils::linkColor()
 {
     const auto theme = UserSettings::instance()->theme();
 
-    if (theme == "light") {
-        return "#0077b5";
-    } else if (theme == "dark") {
-        return "#38A3D8";
+    if (theme == QLatin1String("light")) {
+        return QStringLiteral("#0077b5");
+    } else if (theme == QLatin1String("dark")) {
+        return QStringLiteral("#38A3D8");
     } else {
         return QPalette().color(QPalette::Link).name();
     }
@@ -763,20 +771,17 @@ utils::luminance(const QColor &col)
 }
 
 void
-utils::centerWidget(QWidget *widget, QWidget *parent)
+utils::centerWidget(QWidget *widget, QWindow *parent)
 {
+    if (parent) {
+        widget->window()->windowHandle()->setTransientParent(parent);
+        return;
+    }
+
     auto findCenter = [childRect = widget->rect()](QRect hostRect) -> QPoint {
         return QPoint(hostRect.center().x() - (childRect.width() * 0.5),
                       hostRect.center().y() - (childRect.height() * 0.5));
     };
-
-    if (parent) {
-        widget->move(parent->window()->frameGeometry().topLeft() +
-                     parent->window()->rect().center() - widget->rect().center());
-        return;
-    }
-
-    // Deprecated in 5.13: widget->move(findCenter(QApplication::desktop()->screenGeometry()));
     widget->move(findCenter(QGuiApplication::primaryScreen()->geometry()));
 }
 
