@@ -1,4 +1,5 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
+// SPDX-FileCopyrightText: 2022 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -94,8 +95,8 @@ event_body(const mtx::events::collections::TimelineEvents &event);
 //! Match widgets/events with a description message.
 template<class T>
 QString
-messageDescription(const QString &username = "",
-                   const QString &body     = "",
+messageDescription(const QString &username = QString(),
+                   const QString &body     = QString(),
                    const bool isLocal      = false)
 {
     using Audio      = mtx::events::RoomEvent<mtx::events::msg::Audio>;
@@ -155,10 +156,9 @@ messageDescription(const QString &username = "",
             return QCoreApplication::translate("message-description sent:", "You: %1").arg(body);
         else
             return QCoreApplication::translate("message-description sent:", "%1: %2")
-              .arg(username)
-              .arg(body);
+              .arg(username, body);
     } else if (std::is_same<T, Emote>::value) {
-        return QString("* %1 %2").arg(username).arg(body);
+        return QStringLiteral("* %1 %2").arg(username, body);
     } else if (std::is_same<T, Encrypted>::value) {
         if (isLocal)
             return QCoreApplication::translate("message-description sent:",
@@ -290,7 +290,7 @@ luminance(const QColor &col);
 
 //! Center a widget in relation to another widget.
 void
-centerWidget(QWidget *widget, QWidget *parent);
+centerWidget(QWidget *widget, QWindow *parent);
 
 void
 restoreCombobox(QComboBox *combo, const QString &value);
