@@ -1,9 +1,11 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
+// SPDX-FileCopyrightText: 2022 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #pragma once
 
+#include <QDialogButtonBox>
 #include <QFrame>
 
 #include <mtx/requests.hpp>
@@ -36,7 +38,7 @@ private:
     Toggle *directToggle_;
 
     QPushButton *confirmBtn_;
-    QPushButton *cancelBtn_;
+    QDialogButtonBox *buttonBox_;
 
     TextField *nameInput_;
     TextField *topicInput_;
