@@ -1,4 +1,5 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
+// SPDX-FileCopyrightText: 2022 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -429,23 +430,23 @@ DeviceVerificationFlow::state()
 {
     switch (state_) {
     case PromptStartVerification:
-        return "PromptStartVerification";
+        return QStringLiteral("PromptStartVerification");
     case CompareEmoji:
-        return "CompareEmoji";
+        return QStringLiteral("CompareEmoji");
     case CompareNumber:
-        return "CompareNumber";
+        return QStringLiteral("CompareNumber");
     case WaitingForKeys:
-        return "WaitingForKeys";
+        return QStringLiteral("WaitingForKeys");
     case WaitingForOtherToAccept:
-        return "WaitingForOtherToAccept";
+        return QStringLiteral("WaitingForOtherToAccept");
     case WaitingForMac:
-        return "WaitingForMac";
+        return QStringLiteral("WaitingForMac");
     case Success:
-        return "Success";
+        return QStringLiteral("Success");
     case Failed:
-        return "Failed";
+        return QStringLiteral("Failed");
     default:
-        return "";
+        return QString();
     }
 }
 
