@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2012 Roland Hieber <rohieb@rohieb.name>
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
+// SPDX-FileCopyrightText: 2022 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -24,23 +25,26 @@
 #include "Cache.h"
 #include "EventAccessors.h"
 #include "MxcImageProvider.h"
+#include "UserSettingsPage.h"
 #include "Utils.h"
 
 NotificationsManager::NotificationsManager(QObject *parent)
   : QObject(parent)
-  , dbus("org.freedesktop.Notifications",
-         "/org/freedesktop/Notifications",
-         "org.freedesktop.Notifications",
+  , dbus(QStringLiteral("org.freedesktop.Notifications"),
+         QStringLiteral("/org/freedesktop/Notifications"),
+         QStringLiteral("org.freedesktop.Notifications"),
          QDBusConnection::sessionBus(),
          this)
   , hasMarkup_{std::invoke([this]() -> bool {
-      for (auto x : dbus.call("GetCapabilities").arguments())
+      auto caps = dbus.call("GetCapabilities").arguments();
+      for (const auto &x : qAsConst(caps))
           if (x.toStringList().contains("body-markup"))
               return true;
       return false;
   })}
   , hasImages_{std::invoke([this]() -> bool {
-      for (auto x : dbus.call("GetCapabilities").arguments())
+      auto caps = dbus.call("GetCapabilities").arguments();
+      for (const auto &x : qAsConst(caps))
           if (x.toStringList().contains("body-images"))
               return true;
       return false;
@@ -48,24 +52,26 @@ NotificationsManager::NotificationsManager(QObject *parent)
 {
     qDBusRegisterMetaType<QImage>();
 
-    QDBusConnection::sessionBus().connect("org.freedesktop.Notifications",
-                                          "/org/freedesktop/Notifications",
-                                          "org.freedesktop.Notifications",
-                                          "ActionInvoked",
+    // clang-format off
+    QDBusConnection::sessionBus().connect(QStringLiteral("org.freedesktop.Notifications"),
+                                          QStringLiteral("/org/freedesktop/Notifications"),
+                                          QStringLiteral("org.freedesktop.Notifications"),
+                                          QStringLiteral("ActionInvoked"),
                                           this,
-                                          SLOT(actionInvoked(uint, QString)));
-    QDBusConnection::sessionBus().connect("org.freedesktop.Notifications",
-                                          "/org/freedesktop/Notifications",
-                                          "org.freedesktop.Notifications",
-                                          "NotificationClosed",
+                                          SLOT(actionInvoked(uint,QString)));
+    QDBusConnection::sessionBus().connect(QStringLiteral("org.freedesktop.Notifications"),
+                                          QStringLiteral("/org/freedesktop/Notifications"),
+                                          QStringLiteral("org.freedesktop.Notifications"),
+                                          QStringLiteral("NotificationClosed"),
                                           this,
-                                          SLOT(notificationClosed(uint, uint)));
-    QDBusConnection::sessionBus().connect("org.freedesktop.Notifications",
-                                          "/org/freedesktop/Notifications",
-                                          "org.freedesktop.Notifications",
-                                          "NotificationReplied",
+                                          SLOT(notificationClosed(uint,uint)));
+    QDBusConnection::sessionBus().connect(QStringLiteral("org.freedesktop.Notifications"),
+                                          QStringLiteral("/org/freedesktop/Notifications"),
+                                          QStringLiteral("org.freedesktop.Notifications"),
+                                          QStringLiteral("NotificationReplied"),
                                           this,
-                                          SLOT(notificationReplied(uint, QString)));
+                                          SLOT(notificationReplied(uint,QString)));
+    // clang-format on
 
     connect(this,
             &NotificationsManager::systemPostNotificationCb,
@@ -82,8 +88,14 @@ NotificationsManager::postNotification(const mtx::responses::Notification &notif
     const auto event_id  = QString::fromStdString(mtx::accessors::event_id(notification.event));
     const auto room_name = QString::fromStdString(cache::singleRoomInfo(notification.room_id).name);
 
-    auto postNotif = [this, room_id, event_id, room_name, icon](QString text) {
-        emit systemPostNotificationCb(room_id, event_id, room_name, text, icon);
+    const auto replaces_event_id =
+      QString::fromStdString(mtx::accessors::relations(notification.event).replaces().value_or(""));
+
+    auto postNotif = [this, room_id, event_id, room_name, icon, replaces_event_id](QString text) {
+        if (replaces_event_id.isEmpty())
+            emit systemPostNotificationCb(room_id, event_id, room_name, text, icon);
+        else
+            emit systemPostNotificationCb(room_id, replaces_event_id, room_name, text, icon);
     };
 
     QString template_ = getMessageTemplate(notification);
@@ -98,17 +110,18 @@ NotificationsManager::postNotification(const mtx::responses::Notification &notif
         if (hasImages_ &&
             mtx::accessors::msg_type(notification.event) == mtx::events::MessageType::Image) {
             MxcImageProvider::download(
-              QString::fromStdString(mtx::accessors::url(notification.event)).remove("mxc://"),
+              QString::fromStdString(mtx::accessors::url(notification.event))
+                .remove(QStringLiteral("mxc://")),
               QSize(200, 80),
               [postNotif, notification, template_](QString, QSize, QImage, QString imgPath) {
                   if (imgPath.isEmpty())
                       postNotif(template_
                                   .arg(utils::stripReplyFallbacks(notification.event, {}, {})
                                          .quoted_formatted_body)
-                                  .replace("<em>", "<i>")
-                                  .replace("</em>", "</i>")
-                                  .replace("<strong>", "<b>")
-                                  .replace("</strong>", "</b>"));
+                                  .replace(QLatin1String("<em>"), QLatin1String("<i>"))
+                                  .replace(QLatin1String("</em>"), QLatin1String("</i>"))
+                                  .replace(QLatin1String("<strong>"), QLatin1String("<b>"))
+                                  .replace(QLatin1String("</strong>"), QLatin1String("</b>")));
                   else
                       postNotif(template_.arg(
                         QStringLiteral("<br><img src=\"file:///") % imgPath % "\" alt=\"" %
@@ -120,10 +133,10 @@ NotificationsManager::postNotification(const mtx::responses::Notification &notif
         postNotif(
           template_
             .arg(utils::stripReplyFallbacks(notification.event, {}, {}).quoted_formatted_body)
-            .replace("<em>", "<i>")
-            .replace("</em>", "</i>")
-            .replace("<strong>", "<b>")
-            .replace("</strong>", "</b>"));
+            .replace(QLatin1String("<em>"), QLatin1String("<i>"))
+            .replace(QLatin1String("</em>"), QLatin1String("</i>"))
+            .replace(QLatin1String("<strong>"), QLatin1String("<b>"))
+            .replace(QLatin1String("</strong>"), QLatin1String("</b>")));
         return;
     }
 
@@ -144,28 +157,44 @@ NotificationsManager::systemPostNotification(const QString &room_id,
                                              const QImage &icon)
 {
     QVariantMap hints;
-    hints["image-data"]    = icon;
-    hints["sound-name"]    = "message-new-instant";
-    hints["desktop-entry"] = "nheko";
-    hints["category"]      = "im.received";
+    hints[QStringLiteral("image-data")]    = icon;
+    hints[QStringLiteral("sound-name")]    = "message-new-instant";
+    hints[QStringLiteral("desktop-entry")] = "nheko";
+    hints[QStringLiteral("category")]      = "im.received";
+
+    if (auto profile = UserSettings::instance()->profile(); !profile.isEmpty())
+        hints[QStringLiteral("x-kde-origin-name")] = profile;
+
+    uint replace_id = 0;
+    if (!event_id.isEmpty()) {
+        for (auto elem = notificationIds.begin(); elem != notificationIds.end(); ++elem) {
+            if (elem.value().roomId != room_id)
+                continue;
+
+            if (elem.value().eventId == event_id) {
+                replace_id = elem.key();
+                break;
+            }
+        }
+    }
 
     QList<QVariant> argumentList;
-    argumentList << "nheko";  // app_name
-    argumentList << (uint)0;  // replace_id
-    argumentList << "";       // app_icon
-    argumentList << roomName; // summary
-    argumentList << text;     // body
+    argumentList << "nheko";          // app_name
+    argumentList << (uint)replace_id; // replace_id
+    argumentList << "";               // app_icon
+    argumentList << roomName;         // summary
+    argumentList << text;             // body
 
     // The list of actions has always the action name and then a localized version of that
     // action. Currently we just use an empty string for that.
     // TODO(Nico): Look into what to actually put there.
-    argumentList << (QStringList("default") << ""
-                                            << "inline-reply"
-                                            << ""); // actions
-    argumentList << hints;                          // hints
-    argumentList << (int)-1;                        // timeout in ms
+    argumentList << (QStringList(QStringLiteral("default"))
+                     << QLatin1String("") << QStringLiteral("inline-reply")
+                     << QLatin1String("")); // actions
+    argumentList << hints;                  // hints
+    argumentList << (int)-1;                // timeout in ms
 
-    QDBusPendingCall call = dbus.asyncCallWithArgumentList("Notify", argumentList);
+    QDBusPendingCall call = dbus.asyncCallWithArgumentList(QStringLiteral("Notify"), argumentList);
     auto watcher          = new QDBusPendingCallWatcher{call, this};
     connect(
       watcher, &QDBusPendingCallWatcher::finished, this, [watcher, this, room_id, event_id]() {
@@ -181,7 +210,7 @@ NotificationsManager::systemPostNotification(const QString &room_id,
 void
 NotificationsManager::closeNotification(uint id)
 {
-    auto call    = dbus.asyncCall("CloseNotification", (uint)id); // replace_id
+    auto call    = dbus.asyncCall(QStringLiteral("CloseNotification"), (uint)id); // replace_id
     auto watcher = new QDBusPendingCallWatcher{call, this};
     connect(watcher, &QDBusPendingCallWatcher::finished, this, [watcher]() {
         if (watcher->reply().type() == QDBusMessage::ErrorMessage) {
@@ -219,7 +248,7 @@ NotificationsManager::actionInvoked(uint id, QString action)
 {
     if (notificationIds.contains(id)) {
         roomEventId idEntry = notificationIds[id];
-        if (action == "default") {
+        if (action == QLatin1String("default")) {
             emit notificationClicked(idEntry.roomId, idEntry.eventId);
         }
     }
@@ -260,33 +289,17 @@ operator<<(QDBusArgument &arg, const QImage &image)
         return arg;
     }
 
-    QImage scaled = image.scaledToHeight(100, Qt::SmoothTransformation);
-    scaled        = scaled.convertToFormat(QImage::Format_ARGB32);
-
-#if Q_BYTE_ORDER == Q_LITTLE_ENDIAN
-    // ABGR -> ARGB
-    QImage i = scaled.rgbSwapped();
-#else
-    // ABGR -> GBAR
-    QImage i(scaled.size(), scaled.format());
-    for (int y = 0; y < i.height(); ++y) {
-        QRgb *p   = (QRgb *)scaled.scanLine(y);
-        QRgb *q   = (QRgb *)i.scanLine(y);
-        QRgb *end = p + scaled.width();
-        while (p < end) {
-            *q = qRgba(qGreen(*p), qBlue(*p), qAlpha(*p), qRed(*p));
-            p++;
-            q++;
-        }
-    }
-#endif
+    QImage i = image.height() > 100 || image.width() > 100
+                 ? image.scaledToHeight(100, Qt::SmoothTransformation)
+                 : image;
+    i        = std::move(i).convertToFormat(QImage::Format_RGBA8888);
 
     arg.beginStructure();
     arg << i.width();
     arg << i.height();
     arg << i.bytesPerLine();
     arg << i.hasAlphaChannel();
-    int channels = i.isGrayscale() ? 1 : (i.hasAlphaChannel() ? 4 : 3);
+    int channels = i.hasAlphaChannel() ? 4 : 3;
     arg << i.depth() / channels;
     arg << channels;
     arg << QByteArray(reinterpret_cast<const char *>(i.bits()), i.sizeInBytes());
