@@ -1,4 +1,5 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
+// SPDX-FileCopyrightText: 2022 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -15,6 +16,7 @@
 #include "EventAccessors.h"
 #include "Logging.h"
 #include "MatrixClient.h"
+#include "UserSettingsPage.h"
 #include "Utils.h"
 
 Q_DECLARE_METATYPE(Reaction)
@@ -106,9 +108,9 @@ EventStore::EventStore(std::string room_id, QObject *)
         }
 
         std::visit(
-          [this](auto e) {
-              auto txn_id       = e.event_id;
-              this->current_txn = txn_id;
+          [this](const auto &e) {
+              const auto &txn_id = e.event_id;
+              this->current_txn  = txn_id;
 
               if (txn_id.empty() || txn_id[0] != 'm') {
                   nhlog::ui()->debug("Invalid txn id '{}'", txn_id);
@@ -182,48 +184,95 @@ EventStore::EventStore(std::string room_id, QObject *)
           // Replace the event_id in pending edits/replies/redactions with the actual
           // event_id of this event. This allows one to edit and reply to events that are
           // currently pending.
+          for (const auto &pending_event_id : cache::client()->pendingEvents(room_id_)) {
+              if (auto pending_event = cache::client()->getEvent(room_id_, pending_event_id)) {
+                  bool was_encrypted = false;
+                  mtx::events::EncryptedEvent<mtx::events::msg::Encrypted> original_encrypted;
+                  if (auto encrypted =
+                        std::get_if<mtx::events::EncryptedEvent<mtx::events::msg::Encrypted>>(
+                          &pending_event->data)) {
+                      auto d_event = decryptEvent({room_id_, encrypted->event_id}, *encrypted);
+                      if (d_event->event) {
+                          was_encrypted       = true;
+                          original_encrypted  = *encrypted;
+                          pending_event->data = *d_event->event;
+                      }
+                  }
 
-          // FIXME (introduced by balsoft): this doesn't work for encrypted events, but
-          // allegedly it's hard to fix so I'll leave my first contribution at that
-          for (auto related_event_id : cache::client()->relatedEvents(room_id_, txn_id)) {
-              if (cache::client()->getEvent(room_id_, related_event_id)) {
-                  auto related_event =
-                    cache::client()->getEvent(room_id_, related_event_id).value();
-                  auto relations = mtx::accessors::relations(related_event.data);
+                  auto relations = mtx::accessors::relations(pending_event->data);
 
                   // Replace the blockquote in fallback reply
                   auto related_text = std::get_if<mtx::events::RoomEvent<mtx::events::msg::Text>>(
-                    &related_event.data);
+                    &pending_event->data);
                   if (related_text && relations.reply_to() == txn_id) {
                       size_t index = related_text->content.formatted_body.find(txn_id);
                       if (index != std::string::npos) {
                           related_text->content.formatted_body.replace(
-                            index, event_id.length(), event_id);
+                            index, txn_id.length(), event_id);
                       }
                   }
 
+                  bool replaced_txn = false;
                   for (mtx::common::Relation &rel : relations.relations) {
-                      if (rel.event_id == txn_id)
+                      if (rel.event_id == txn_id) {
                           rel.event_id = event_id;
+                          replaced_txn = true;
+                      }
                   }
 
-                  mtx::accessors::set_relations(related_event.data, relations);
+                  if (!replaced_txn)
+                      continue;
 
-                  cache::client()->replaceEvent(room_id_, related_event_id, related_event);
+                  mtx::accessors::set_relations(pending_event->data, std::move(relations));
 
-                  auto idx = idToIndex(related_event_id);
+                  // reencrypt. This is a bit of a hack and might make people able to read the
+                  // message, that were in the room at the time of sending the last pending message.
+                  // That window is pretty small though, so it should be good enough. We also just
+                  // fail, if there was no session. But there SHOULD always be one. Let's wait until
+                  // I am proven wrong :3
+                  if (was_encrypted) {
+                      auto session = cache::getOutboundMegolmSession(room_id_);
+                      if (!session.session)
+                          continue;
 
-                  events_by_id_.remove({room_id_, related_event_id});
+                      std::visit(
+                        [&pending_event, &original_encrypted, &session, this](auto &msg) {
+                            json doc = {{"type", mtx::events::to_string(msg.type)},
+                                        {"content", json(msg.content)},
+                                        {"room_id", room_id_}};
+
+                            auto data = olm::encrypt_group_message_with_session(
+                              session.session, http::client()->device_id(), doc);
+
+                            session.data.message_index =
+                              olm_outbound_group_session_message_index(session.session.get());
+                            cache::updateOutboundMegolmSession(
+                              room_id_, session.data, session.session);
+
+                            original_encrypted.content = data;
+                            pending_event->data        = original_encrypted;
+                        },
+                        pending_event->data);
+                  }
+
+                  cache::client()->replaceEvent(room_id_, pending_event_id, *pending_event);
+
+                  auto idx = idToIndex(pending_event_id);
+
+                  events_by_id_.remove({room_id_, pending_event_id});
                   events_.remove({room_id_, toInternalIdx(*idx)});
               }
           }
 
           http::client()->read_event(
-            room_id_, event_id, [this, event_id](mtx::http::RequestErr err) {
+            room_id_,
+            event_id,
+            [this, event_id](mtx::http::RequestErr err) {
                 if (err) {
                     nhlog::net()->warn("failed to read_event ({}, {})", room_id_, event_id);
                 }
-            });
+            },
+            !UserSettings::instance()->readReceipts());
 
           auto idx = idToIndex(event_id);
 
@@ -451,8 +500,8 @@ EventStore::edits(const std::string &event_id)
         std::holds_alternative<mtx::events::RoomEvent<mtx::events::msg::Redacted>>(*original_event))
         return {};
 
-    auto original_sender    = mtx::accessors::sender(*original_event);
-    auto original_relations = mtx::accessors::relations(*original_event);
+    const auto &original_sender    = mtx::accessors::sender(*original_event);
+    const auto &original_relations = mtx::accessors::relations(*original_event);
 
     std::vector<mtx::events::collections::TimelineEvents> edits;
     for (const auto &id : event_ids) {
@@ -460,15 +509,15 @@ EventStore::edits(const std::string &event_id)
         if (!related_event)
             continue;
 
-        auto related_ev = *related_event;
-
-        auto edit_rel = mtx::accessors::relations(related_ev);
+        const auto &edit_rel = mtx::accessors::relations(*related_event);
         if (edit_rel.replaces() == event_id &&
-            original_sender == mtx::accessors::sender(related_ev)) {
+            original_sender == mtx::accessors::sender(*related_event)) {
+            auto related_ev = *related_event;
             if (edit_rel.synthesized && original_relations.reply_to() && !edit_rel.reply_to()) {
-                edit_rel.relations.push_back(
+                auto edit_rel_copy = edit_rel;
+                edit_rel_copy.relations.push_back(
                   {mtx::common::RelationType::InReplyTo, original_relations.reply_to().value()});
-                mtx::accessors::set_relations(related_ev, std::move(edit_rel));
+                mtx::accessors::set_relations(related_ev, std::move(edit_rel_copy));
             }
             edits.push_back(std::move(related_ev));
         }
@@ -527,6 +576,7 @@ EventStore::reactions(const std::string &event_id)
     }
 
     QVariantList temp;
+    temp.reserve(static_cast<int>(reactions.size()));
     for (auto &reaction : reactions) {
         const auto &agg            = aggregation[reaction.key_.toStdString()];
         reaction.count_            = agg.count;
@@ -537,15 +587,11 @@ EventStore::reactions(const std::string &event_id)
             if (firstReaction)
                 firstReaction = false;
             else
-                reaction.users_ += ", ";
+                reaction.users_ += QLatin1String(", ");
 
             reaction.users_ += QString::fromStdString(user);
         }
 
-        nhlog::db()->debug("key: {}, count: {}, users: {}",
-                           reaction.key_.toStdString(),
-                           reaction.count_,
-                           reaction.users_.toStdString());
         temp.append(QVariant::fromValue(reaction));
     }
 
@@ -674,6 +720,9 @@ EventStore::decryptEvent(const IdIndex &idx,
     }
 
     auto encInfo = mtx::accessors::file(decryptionResult.event.value());
+    if (encInfo)
+        emit newEncryptedImage(encInfo.value());
+    encInfo = mtx::accessors::thumbnail_file(decryptionResult.event.value());
     if (encInfo)
         emit newEncryptedImage(encInfo.value());
 
@@ -717,7 +766,8 @@ void
 EventStore::enableKeyRequests(bool suppressKeyRequests_)
 {
     if (!suppressKeyRequests_) {
-        for (const auto &key : decryptedEvents_.keys())
+        auto keys = decryptedEvents_.keys();
+        for (const auto &key : qAsConst(keys))
             if (key.room == this->room_id_)
                 decryptedEvents_.remove(key);
         suppressKeyRequests = false;
@@ -726,7 +776,10 @@ EventStore::enableKeyRequests(bool suppressKeyRequests_)
 }
 
 mtx::events::collections::TimelineEvents *
-EventStore::get(std::string id, std::string_view related_to, bool decrypt, bool resolve_edits)
+EventStore::get(const std::string &id,
+                std::string_view related_to,
+                bool decrypt,
+                bool resolve_edits)
 {
     if (this->thread() != QThread::currentThread())
         nhlog::db()->warn("{} called from a different thread!", __func__);
@@ -734,7 +787,7 @@ EventStore::get(std::string id, std::string_view related_to, bool decrypt, bool
     if (id.empty())
         return nullptr;
 
-    IdIndex index{room_id_, std::move(id)};
+    IdIndex index{room_id_, id};
     if (resolve_edits) {
         auto edits_ = edits(index.id);
         if (!edits_.empty()) {
