@@ -1,4 +1,5 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
+// SPDX-FileCopyrightText: 2022 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -7,6 +8,7 @@
 #include "Cache_p.h"
 #include "ChatPage.h"
 #include "Logging.h"
+#include "MainWindow.h"
 #include "MatrixClient.h"
 #include "MxcImageProvider.h"
 #include "TimelineModel.h"
@@ -90,6 +92,7 @@ RoomlistModel::data(const QModelIndex &index, int role) const
         if (role == Roles::ParentSpaces) {
             auto parents = cache::client()->getParentRoomIds(roomid.toStdString());
             QStringList list;
+            list.reserve(static_cast<int>(parents.size()));
             for (const auto &t : parents)
                 list.push_back(QString::fromStdString(t));
             return list;
@@ -98,7 +101,8 @@ RoomlistModel::data(const QModelIndex &index, int role) const
         } else if (role == Roles::IsDirect) {
             return directChatToUser.count(roomid) > 0;
         } else if (role == Roles::DirectChatOtherUserId) {
-            return directChatToUser.count(roomid) ? directChatToUser.at(roomid).front() : "";
+            return directChatToUser.count(roomid) ? directChatToUser.at(roomid).front()
+                                                  : QLatin1String("");
         }
 
         if (models.contains(roomid)) {
@@ -113,7 +117,7 @@ RoomlistModel::data(const QModelIndex &index, int role) const
             case Roles::Time:
                 return room->lastMessage().descriptiveTime;
             case Roles::Timestamp:
-                return QVariant(static_cast<quint64>(room->lastMessage().timestamp));
+                return QVariant{static_cast<quint64>(room->lastMessage().timestamp)};
             case Roles::HasUnreadMessages:
                 return this->roomReadStatus.count(roomid) && this->roomReadStatus.at(roomid);
             case Roles::HasLoudNotification:
@@ -129,6 +133,7 @@ RoomlistModel::data(const QModelIndex &index, int role) const
             case Roles::Tags: {
                 auto info = cache::singleRoomInfo(roomid.toStdString());
                 QStringList list;
+                list.reserve(static_cast<int>(info.tags.size()));
                 for (const auto &t : info.tags)
                     list.push_back(QString::fromStdString(t));
                 return list;
@@ -148,7 +153,7 @@ RoomlistModel::data(const QModelIndex &index, int role) const
             case Roles::Time:
                 return QString();
             case Roles::Timestamp:
-                return QVariant(static_cast<quint64>(0));
+                return QVariant{static_cast<quint64>(0)};
             case Roles::HasUnreadMessages:
             case Roles::HasLoudNotification:
                 return false;
@@ -177,7 +182,7 @@ RoomlistModel::data(const QModelIndex &index, int role) const
             case Roles::Time:
                 return QString();
             case Roles::Timestamp:
-                return QVariant(static_cast<quint64>(0));
+                return QVariant{static_cast<quint64>(0)};
             case Roles::HasUnreadMessages:
             case Roles::HasLoudNotification:
                 return false;
@@ -202,7 +207,6 @@ RoomlistModel::data(const QModelIndex &index, int role) const
             else if (role == Roles::IsPreviewFetched)
                 return false;
 
-            fetchPreview(roomid);
             switch (role) {
             case Roles::AvatarUrl:
                 return QString();
@@ -271,7 +275,7 @@ RoomlistModel::addRoom(const QString &room_id, bool suppressInsertNotification)
 
         connect(newRoom.data(),
                 &TimelineModel::newEncryptedImage,
-                manager->imageProvider(),
+                MainWindow::instance()->imageProvider(),
                 &MxcImageProvider::addEncryptionInfo);
         connect(newRoom.data(),
                 &TimelineModel::forwardToRoom,
@@ -317,7 +321,7 @@ RoomlistModel::addRoom(const QString &room_id, bool suppressInsertNotification)
 
             int total_unread_msgs = 0;
 
-            for (const auto &room : models) {
+            for (const auto &room : qAsConst(models)) {
                 if (!room.isNull())
                     total_unread_msgs += room->notificationCount();
             }
@@ -381,56 +385,60 @@ RoomlistModel::addRoom(const QString &room_id, bool suppressInsertNotification)
 }
 
 void
-RoomlistModel::fetchPreview(QString roomid_) const
+RoomlistModel::fetchPreviews(QString roomid_, const std::string &from)
 {
-    std::string roomid = roomid_.toStdString();
-    http::client()->get_state_event<mtx::events::state::Create>(
-      roomid, "", [this, roomid](const mtx::events::state::Create &c, mtx::http::RequestErr err) {
-          bool is_space = false;
-          if (!err) {
-              is_space = c.type == mtx::events::state::room_type::space;
+    auto roomid = roomid_.toStdString();
+    if (from.empty()) {
+        // check if we need to fetch anything
+        auto children = cache::client()->getChildRoomIds(roomid);
+        bool fetch    = false;
+        for (const auto &c : children) {
+            auto id = QString::fromStdString(c);
+            if (invites.contains(id) || models.contains(id) ||
+                (previewedRooms.contains(id) && previewedRooms.value(id).has_value()))
+                continue;
+            else {
+                fetch = true;
+                break;
+            }
+        }
+        if (!fetch) {
+            nhlog::net()->info("Not feching previews for children of {}", roomid);
+            return;
+        }
+    }
+
+    nhlog::net()->info("Feching previews for children of {}", roomid);
+    http::client()->get_hierarchy(
+      roomid,
+      [this, roomid, roomid_](const mtx::responses::HierarchyRooms &h, mtx::http::RequestErr err) {
+          if (err) {
+              nhlog::net()->error("Failed to fetch previews for children of {}: {}", roomid, *err);
+              return;
           }
 
-          http::client()->get_state_event<mtx::events::state::Avatar>(
-            roomid,
-            "",
-            [this, roomid, is_space](const mtx::events::state::Avatar &a, mtx::http::RequestErr) {
-                auto avatar_url = a.url;
+          nhlog::net()->info("Feched previews for children of {}: {}", roomid, h.rooms.size());
 
-                http::client()->get_state_event<mtx::events::state::Topic>(
-                  roomid,
-                  "",
-                  [this, roomid, avatar_url, is_space](const mtx::events::state::Topic &t,
-                                                       mtx::http::RequestErr) {
-                      auto topic = t.topic;
-                      http::client()->get_state_event<mtx::events::state::Name>(
-                        roomid,
-                        "",
-                        [this, roomid, topic, avatar_url, is_space](
-                          const mtx::events::state::Name &n, mtx::http::RequestErr err) {
-                            if (err) {
-                                nhlog::net()->warn("Failed to fetch name event to "
-                                                   "create preview for {}",
-                                                   roomid);
-                            }
+          for (const auto &e : h.rooms) {
+              RoomInfo info{};
+              info.name         = e.name;
+              info.is_space     = e.room_type == mtx::events::state::room_type::space;
+              info.avatar_url   = e.avatar_url;
+              info.topic        = e.topic;
+              info.guest_access = e.guest_can_join;
+              info.join_rule    = e.join_rule;
+              info.member_count = e.num_joined_members;
 
-                            // don't even add a preview, if we got not a single
-                            // response
-                            if (n.name.empty() && avatar_url.empty() && topic.empty())
-                                return;
+              emit fetchedPreview(QString::fromStdString(e.room_id), info);
+          }
 
-                            RoomInfo info{};
-                            info.name       = n.name;
-                            info.is_space   = is_space;
-                            info.avatar_url = avatar_url;
-                            info.topic      = topic;
-
-                            const_cast<RoomlistModel *>(this)->fetchedPreview(
-                              QString::fromStdString(roomid), info);
-                        });
-                  });
-            });
-      });
+          if (!h.next_batch.empty())
+              fetchPreviews(roomid_, h.next_batch);
+      },
+      from,
+      50,
+      1,
+      false);
 }
 
 std::set<QString>
@@ -505,7 +513,7 @@ RoomlistModel::sync(const mtx::responses::Sync &sync_)
         // room_model->addEvents(room.timeline);
         connect(room_model.data(),
                 &TimelineModel::newCallEvent,
-                manager->callManager(),
+                ChatPage::instance()->callManager(),
                 &CallManager::syncEvent,
                 Qt::UniqueConnection);
 
@@ -586,8 +594,8 @@ RoomlistModel::initializeRooms()
     }
 
     invites = cache::client()->invites();
-    for (const auto &id : invites.keys()) {
-        roomids.push_back(id);
+    for (auto id = invites.keyBegin(); id != invites.keyEnd(); ++id) {
+        roomids.push_back(*id);
     }
 
     for (const auto &id : cache::client()->roomIds())
@@ -826,12 +834,14 @@ FilteredRoomlistModel::updateHiddenTagsAndSpaces()
     hiddenTags.clear();
     hiddenSpaces.clear();
     hideDMs = false;
-    for (const auto &t : UserSettings::instance()->hiddenTags()) {
-        if (t.startsWith("tag:"))
+
+    auto hidden = UserSettings::instance()->hiddenTags();
+    for (const auto &t : qAsConst(hidden)) {
+        if (t.startsWith(u"tag:"))
             hiddenTags.push_back(t.mid(4));
-        else if (t.startsWith("space:"))
+        else if (t.startsWith(u"space:"))
             hiddenSpaces.push_back(t.mid(6));
-        else if (t == "dm")
+        else if (t == QLatin1String("dm"))
             hideDMs = true;
     }
 
