@@ -1,4 +1,5 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
+// SPDX-FileCopyrightText: 2022 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -90,6 +91,8 @@ enum EventType
     Sticker,
     // m.tag
     Tag,
+    // m.widget
+    Widget,
     /// m.room.message
     AudioMessage,
     EmoteMessage,
@@ -122,6 +125,8 @@ enum EventType
 };
 Q_ENUM_NS(EventType)
 mtx::events::EventType fromRoomEventType(qml_mtx_events::EventType);
+qml_mtx_events::EventType
+toRoomEventType(mtx::events::EventType e);
 
 enum EventState
 {
@@ -177,6 +182,7 @@ class TimelineModel : public QAbstractListModel
     Q_PROPERTY(QString roomAvatarUrl READ roomAvatarUrl NOTIFY roomAvatarUrlChanged)
     Q_PROPERTY(QString roomTopic READ roomTopic NOTIFY roomTopicChanged)
     Q_PROPERTY(QStringList pinnedMessages READ pinnedMessages NOTIFY pinnedMessagesChanged)
+    Q_PROPERTY(QStringList widgetLinks READ widgetLinks NOTIFY widgetLinksChanged)
     Q_PROPERTY(int roomMemberCount READ roomMemberCount NOTIFY roomMemberCountChanged)
     Q_PROPERTY(bool isEncrypted READ isEncrypted NOTIFY encryptionChanged)
     Q_PROPERTY(bool isSpace READ isSpace CONSTANT)
@@ -204,10 +210,12 @@ public:
         UserId,
         UserName,
         PreviousMessageDay,
+        PreviousMessageIsStateEvent,
         Day,
         Timestamp,
         Url,
         ThumbnailUrl,
+        Duration,
         Blurhash,
         Filename,
         Filesize,
@@ -220,6 +228,7 @@ public:
         IsEdited,
         IsEditable,
         IsEncrypted,
+        IsStateEvent,
         Trustlevel,
         EncryptionError,
         ReplyTo,
@@ -237,54 +246,56 @@ public:
     int rowCount(const QModelIndex &parent = QModelIndex()) const override;
     QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;
     QVariant data(const mtx::events::collections::TimelineEvents &event, int role) const;
-    Q_INVOKABLE QVariant dataById(QString id, int role, QString relatedTo);
+    Q_INVOKABLE QVariant dataById(const QString &id, int role, const QString &relatedTo);
 
     bool canFetchMore(const QModelIndex &) const override;
     void fetchMore(const QModelIndex &) override;
 
-    Q_INVOKABLE QString displayName(QString id) const;
-    Q_INVOKABLE QString avatarUrl(QString id) const;
+    Q_INVOKABLE QString displayName(const QString &id) const;
+    Q_INVOKABLE QString avatarUrl(const QString &id) const;
     Q_INVOKABLE QString formatDateSeparator(QDate date) const;
-    Q_INVOKABLE QString formatTypingUsers(const std::vector<QString> &users, QColor bg);
-    Q_INVOKABLE bool showAcceptKnockButton(QString id);
-    Q_INVOKABLE void acceptKnock(QString id);
-    Q_INVOKABLE QString formatMemberEvent(QString id);
-    Q_INVOKABLE QString formatJoinRuleEvent(QString id);
-    Q_INVOKABLE QString formatHistoryVisibilityEvent(QString id);
-    Q_INVOKABLE QString formatGuestAccessEvent(QString id);
-    Q_INVOKABLE QString formatPowerLevelEvent(QString id);
-    Q_INVOKABLE QVariantMap formatRedactedEvent(QString id);
+    Q_INVOKABLE QString formatTypingUsers(const std::vector<QString> &users, const QColor &bg);
+    Q_INVOKABLE bool showAcceptKnockButton(const QString &id);
+    Q_INVOKABLE void acceptKnock(const QString &id);
+    Q_INVOKABLE QString formatMemberEvent(const QString &id);
+    Q_INVOKABLE QString formatJoinRuleEvent(const QString &id);
+    Q_INVOKABLE QString formatHistoryVisibilityEvent(const QString &id);
+    Q_INVOKABLE QString formatGuestAccessEvent(const QString &id);
+    Q_INVOKABLE QString formatPowerLevelEvent(const QString &id);
+    Q_INVOKABLE QVariantMap formatRedactedEvent(const QString &id);
 
-    Q_INVOKABLE void viewRawMessage(QString id);
-    Q_INVOKABLE void forwardMessage(QString eventId, QString roomId);
-    Q_INVOKABLE void viewDecryptedRawMessage(QString id);
+    Q_INVOKABLE void viewRawMessage(const QString &id);
+    Q_INVOKABLE void forwardMessage(const QString &eventId, QString roomId);
+    Q_INVOKABLE void viewDecryptedRawMessage(const QString &id);
     Q_INVOKABLE void openUserProfile(QString userid);
     Q_INVOKABLE void editAction(QString id);
-    Q_INVOKABLE void replyAction(QString id);
-    Q_INVOKABLE void unpin(QString id);
-    Q_INVOKABLE void pin(QString id);
+    Q_INVOKABLE void replyAction(const QString &id);
+    Q_INVOKABLE void unpin(const QString &id);
+    Q_INVOKABLE void pin(const QString &id);
     Q_INVOKABLE void showReadReceipts(QString id);
-    Q_INVOKABLE void redactEvent(QString id);
-    Q_INVOKABLE int idToIndex(QString id) const;
+    Q_INVOKABLE void redactEvent(const QString &id, const QString &reason = "");
+    Q_INVOKABLE int idToIndex(const QString &id) const;
     Q_INVOKABLE QString indexToId(int index) const;
-    Q_INVOKABLE void openMedia(QString eventId);
-    Q_INVOKABLE void cacheMedia(QString eventId);
-    Q_INVOKABLE bool saveMedia(QString eventId) const;
+    Q_INVOKABLE void openMedia(const QString &eventId);
+    Q_INVOKABLE void cacheMedia(const QString &eventId);
+    Q_INVOKABLE bool saveMedia(const QString &eventId) const;
     Q_INVOKABLE void showEvent(QString eventId);
-    Q_INVOKABLE void copyLinkToEvent(QString eventId) const;
-    void cacheMedia(QString eventId, std::function<void(const QString filename)> callback);
+    Q_INVOKABLE void copyLinkToEvent(const QString &eventId) const;
+    void
+    cacheMedia(const QString &eventId, const std::function<void(const QString filename)> &callback);
     Q_INVOKABLE void sendReset()
     {
         beginResetModel();
         endResetModel();
     }
 
-    Q_INVOKABLE void requestKeyForEvent(QString id);
+    Q_INVOKABLE void requestKeyForEvent(const QString &id);
 
     std::vector<::Reaction> reactions(const std::string &event_id)
     {
         auto list = events.reactions(event_id);
         std::vector<::Reaction> vec;
+        vec.reserve(list.size());
         for (const auto &r : list)
             vec.push_back(r.value<Reaction>());
         return vec;
@@ -296,7 +307,7 @@ public:
     void syncState(const mtx::responses::State &state);
     template<class T>
     void sendMessageEvent(const T &content, mtx::events::EventType eventType);
-    RelatedInfo relatedInfo(QString id);
+    RelatedInfo relatedInfo(const QString &id);
 
     DescInfo lastMessage() const { return lastMessage_; }
     bool isSpace() const { return isSpace_; }
@@ -320,7 +331,7 @@ public slots:
     int currentIndex() const { return idToIndex(currentId); }
     void eventShown();
     void markEventsAsRead(const std::vector<QString> &event_ids);
-    QVariantMap getDump(QString eventId, QString relatedTo) const;
+    QVariantMap getDump(const QString &eventId, const QString &relatedTo) const;
     void updateTypingUsers(const std::vector<QString> &users)
     {
         if (this->typingUsers_ != users) {
@@ -331,7 +342,7 @@ public slots:
     std::vector<QString> typingUsers() const { return typingUsers_; }
     bool paginationInProgress() const { return m_paginationInProgress; }
     QString reply() const { return reply_; }
-    void setReply(QString newReply)
+    void setReply(const QString &newReply)
     {
         if (edit_.startsWith('m'))
             return;
@@ -344,15 +355,16 @@ public slots:
     void resetReply()
     {
         if (!reply_.isEmpty()) {
-            reply_ = "";
+            reply_ = QLatin1String("");
             emit replyChanged(reply_);
         }
     }
     QString edit() const { return edit_; }
-    void setEdit(QString newEdit);
+    void setEdit(const QString &newEdit);
     void resetEdit();
     void setDecryptDescription(bool decrypt) { decryptDescription = decrypt; }
     void clearTimeline() { events.clearTimeline(); }
+    void resetState();
     void receivedSessionKey(const std::string &session_key)
     {
         events.receivedSessionKey(session_key);
@@ -362,13 +374,14 @@ public slots:
     QString plainRoomName() const;
     QString roomTopic() const;
     QStringList pinnedMessages() const;
+    QStringList widgetLinks() const;
     InputBar *input() { return &input_; }
     Permissions *permissions() { return &permissions_; }
     QString roomAvatarUrl() const;
     QString roomId() const { return room_id_; }
 
-    bool hasMentions() { return highlight_count > 0; }
-    int notificationCount() { return notification_count; }
+    bool hasMentions() const { return highlight_count > 0; }
+    int notificationCount() const { return notification_count; }
 
     QString scrollTarget() const;
 
@@ -394,6 +407,8 @@ signals:
     void lastMessageChanged();
     void notificationsChanged();
 
+    void newState(mtx::responses::StateEvents events);
+
     void newMessageToSend(mtx::events::collections::TimelineEvents event);
     void addPendingMessageToStore(mtx::events::collections::TimelineEvents event);
     void updateFlowEventId(std::string event_id);
@@ -404,6 +419,7 @@ signals:
     void plainRoomNameChanged();
     void roomTopicChanged();
     void pinnedMessagesChanged();
+    void widgetLinksChanged();
     void roomAvatarUrlChanged();
     void roomMemberCountChanged();
     void isDirectChanged();
@@ -420,12 +436,12 @@ private:
 
     void setPaginationInProgress(const bool paginationInProgress);
 
+    QString room_id_;
+
     QSet<QString> read;
 
     mutable EventStore events;
 
-    QString room_id_;
-
     QString currentId, currentReadId;
     QString reply_, edit_;
     QString textBeforeEdit, replyBeforeEdit;
