@@ -1,51 +1,29 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
+// SPDX-FileCopyrightText: 2022 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "TimelineViewManager.h"
 
-#include <QDropEvent>
-#include <QMetaType>
-#include <QPalette>
-#include <QQmlContext>
-#include <QQmlEngine>
+#include <QApplication>
+#include <QFileDialog>
+#include <QStandardPaths>
 #include <QString>
 
-#include "BlurhashProvider.h"
 #include "ChatPage.h"
-#include "Clipboard.h"
-#include "ColorImageProvider.h"
 #include "CombinedImagePackModel.h"
 #include "CompletionProxyModel.h"
-#include "DelegateChooser.h"
 #include "EventAccessors.h"
 #include "ImagePackListModel.h"
 #include "InviteesModel.h"
 #include "Logging.h"
 #include "MainWindow.h"
 #include "MatrixClient.h"
-#include "MxcImageProvider.h"
-#include "ReadReceiptsModel.h"
-#include "RoomDirectoryModel.h"
 #include "RoomsModel.h"
-#include "SingleImagePackModel.h"
 #include "UserSettingsPage.h"
 #include "UsersModel.h"
-#include "dialogs/ImageOverlay.h"
 #include "emoji/EmojiModel.h"
 #include "emoji/Provider.h"
-#include "encryption/DeviceVerificationFlow.h"
-#include "encryption/SelfVerificationStatus.h"
-#include "ui/MxcAnimatedImage.h"
-#include "ui/MxcMediaProxy.h"
-#include "ui/NhekoCursorShape.h"
-#include "ui/NhekoDropArea.h"
-#include "ui/NhekoGlobalObject.h"
-#include "ui/UIA.h"
-
-Q_DECLARE_METATYPE(mtx::events::collections::TimelineEvents)
-Q_DECLARE_METATYPE(std::vector<DeviceInfo>)
-Q_DECLARE_METATYPE(std::vector<mtx::responses::PublicRoomsChunk>)
 
 namespace msgs = mtx::events::msg;
 
@@ -100,17 +78,6 @@ void
 TimelineViewManager::updateColorPalette()
 {
     userColors.clear();
-
-    if (ChatPage::instance()->userSettings()->theme() == "light") {
-        view->rootContext()->setContextProperty("currentActivePalette", QPalette());
-        view->rootContext()->setContextProperty("currentInactivePalette", QPalette());
-    } else if (ChatPage::instance()->userSettings()->theme() == "dark") {
-        view->rootContext()->setContextProperty("currentActivePalette", QPalette());
-        view->rootContext()->setContextProperty("currentInactivePalette", QPalette());
-    } else {
-        view->rootContext()->setContextProperty("currentActivePalette", QPalette());
-        view->rootContext()->setContextProperty("currentInactivePalette", nullptr);
-    }
 }
 
 QColor
@@ -122,103 +89,15 @@ TimelineViewManager::userColor(QString id, QColor background)
     return userColors.value(idx);
 }
 
-QString
-TimelineViewManager::userPresence(QString id) const
-{
-    if (id.isEmpty())
-        return "";
-    else
-        return QString::fromStdString(
-          mtx::presence::to_string(cache::presenceState(id.toStdString())));
-}
-
-QString
-TimelineViewManager::userStatus(QString id) const
-{
-    return QString::fromStdString(cache::statusMessage(id.toStdString()));
-}
-
-TimelineViewManager::TimelineViewManager(CallManager *callManager, ChatPage *parent)
+TimelineViewManager::TimelineViewManager(CallManager *, ChatPage *parent)
   : QObject(parent)
-  , imgProvider(new MxcImageProvider())
-  , colorImgProvider(new ColorImageProvider())
-  , blurhashProvider(new BlurhashProvider())
-  , jdenticonProvider(new JdenticonProvider())
   , rooms_(new RoomlistModel(this))
   , communities_(new CommunitiesModel(this))
-  , callManager_(callManager)
   , verificationManager_(new VerificationManager(this))
+  , presenceEmitter(new PresenceEmitter(this))
 {
-    qRegisterMetaType<mtx::events::msg::KeyVerificationAccept>();
-    qRegisterMetaType<mtx::events::msg::KeyVerificationCancel>();
-    qRegisterMetaType<mtx::events::msg::KeyVerificationDone>();
-    qRegisterMetaType<mtx::events::msg::KeyVerificationKey>();
-    qRegisterMetaType<mtx::events::msg::KeyVerificationMac>();
-    qRegisterMetaType<mtx::events::msg::KeyVerificationReady>();
-    qRegisterMetaType<mtx::events::msg::KeyVerificationRequest>();
-    qRegisterMetaType<mtx::events::msg::KeyVerificationStart>();
-    qRegisterMetaType<CombinedImagePackModel *>();
-
-    qRegisterMetaType<std::vector<mtx::responses::PublicRoomsChunk>>();
-
-    qmlRegisterUncreatableMetaObject(
-      qml_mtx_events::staticMetaObject, "im.nheko", 1, 0, "MtxEvent", "Can't instantiate enum!");
-    qmlRegisterUncreatableMetaObject(
-      olm::staticMetaObject, "im.nheko", 1, 0, "Olm", "Can't instantiate enum!");
-    qmlRegisterUncreatableMetaObject(
-      crypto::staticMetaObject, "im.nheko", 1, 0, "Crypto", "Can't instantiate enum!");
-    qmlRegisterUncreatableMetaObject(verification::staticMetaObject,
-                                     "im.nheko",
-                                     1,
-                                     0,
-                                     "VerificationStatus",
-                                     "Can't instantiate enum!");
-
-    qmlRegisterType<DelegateChoice>("im.nheko", 1, 0, "DelegateChoice");
-    qmlRegisterType<DelegateChooser>("im.nheko", 1, 0, "DelegateChooser");
-    qmlRegisterType<NhekoDropArea>("im.nheko", 1, 0, "NhekoDropArea");
-    qmlRegisterType<NhekoCursorShape>("im.nheko", 1, 0, "CursorShape");
-    qmlRegisterType<MxcAnimatedImage>("im.nheko", 1, 0, "MxcAnimatedImage");
-    qmlRegisterType<MxcMediaProxy>("im.nheko", 1, 0, "MxcMedia");
-    qmlRegisterUncreatableType<DeviceVerificationFlow>(
-      "im.nheko", 1, 0, "DeviceVerificationFlow", "Can't create verification flow from QML!");
-    qmlRegisterUncreatableType<UserProfile>(
-      "im.nheko", 1, 0, "UserProfileModel", "UserProfile needs to be instantiated on the C++ side");
-    qmlRegisterUncreatableType<MemberList>(
-      "im.nheko", 1, 0, "MemberList", "MemberList needs to be instantiated on the C++ side");
-    qmlRegisterUncreatableType<RoomSettings>(
-      "im.nheko",
-      1,
-      0,
-      "RoomSettingsModel",
-      "Room Settings needs to be instantiated on the C++ side");
-    qmlRegisterUncreatableType<TimelineModel>(
-      "im.nheko", 1, 0, "Room", "Room needs to be instantiated on the C++ side");
-    qmlRegisterUncreatableType<ImagePackListModel>(
-      "im.nheko",
-      1,
-      0,
-      "ImagePackListModel",
-      "ImagePackListModel needs to be instantiated on the C++ side");
-    qmlRegisterUncreatableType<SingleImagePackModel>(
-      "im.nheko",
-      1,
-      0,
-      "SingleImagePackModel",
-      "SingleImagePackModel needs to be instantiated on the C++ side");
-    qmlRegisterUncreatableType<InviteesModel>(
-      "im.nheko", 1, 0, "InviteesModel", "InviteesModel needs to be instantiated on the C++ side");
-    qmlRegisterUncreatableType<ReadReceiptsProxy>(
-      "im.nheko",
-      1,
-      0,
-      "ReadReceiptsProxy",
-      "ReadReceiptsProxy needs to be instantiated on the C++ side");
-
     static auto self = this;
-    qmlRegisterSingletonInstance("im.nheko", 1, 0, "MainWindow", MainWindow::instance());
     qmlRegisterSingletonInstance("im.nheko", 1, 0, "TimelineManager", self);
-    qmlRegisterSingletonInstance("im.nheko", 1, 0, "UIA", UIA::instance());
     qmlRegisterSingletonType<RoomlistModel>(
       "im.nheko", 1, 0, "Rooms", [](QQmlEngine *, QJSEngine *) -> QObject * {
           auto ptr = new FilteredRoomlistModel(self->rooms_);
@@ -234,79 +113,24 @@ TimelineViewManager::TimelineViewManager(CallManager *callManager, ChatPage *par
           return ptr;
       });
     qmlRegisterSingletonInstance("im.nheko", 1, 0, "Communities", self->communities_);
-    qmlRegisterSingletonInstance(
-      "im.nheko", 1, 0, "Settings", ChatPage::instance()->userSettings().data());
-    qmlRegisterSingletonInstance(
-      "im.nheko", 1, 0, "CallManager", ChatPage::instance()->callManager());
-    qmlRegisterSingletonType<Clipboard>(
-      "im.nheko", 1, 0, "Clipboard", [](QQmlEngine *, QJSEngine *) -> QObject * {
-          return new Clipboard();
-      });
-    qmlRegisterSingletonType<Nheko>(
-      "im.nheko", 1, 0, "Nheko", [](QQmlEngine *, QJSEngine *) -> QObject * {
-          return new Nheko();
-      });
     qmlRegisterSingletonInstance("im.nheko", 1, 0, "VerificationManager", verificationManager_);
-    qmlRegisterSingletonType<SelfVerificationStatus>(
-      "im.nheko", 1, 0, "SelfVerificationStatus", [](QQmlEngine *, QJSEngine *) -> QObject * {
-          auto ptr = new SelfVerificationStatus();
-          QObject::connect(ChatPage::instance(),
-                           &ChatPage::initializeEmptyViews,
-                           ptr,
-                           &SelfVerificationStatus::invalidate);
-          return ptr;
-      });
+    qmlRegisterSingletonInstance("im.nheko", 1, 0, "Presence", presenceEmitter);
 
-    qRegisterMetaType<mtx::events::collections::TimelineEvents>();
-    qRegisterMetaType<std::vector<DeviceInfo>>();
-
-    qmlRegisterUncreatableType<FilteredCommunitiesModel>(
-      "im.nheko",
-      1,
-      0,
-      "FilteredCommunitiesModel",
-      "Use Communities.filtered() to create a FilteredCommunitiesModel");
-
-    qmlRegisterType<emoji::EmojiModel>("im.nheko.EmojiModel", 1, 0, "EmojiModel");
-    qmlRegisterUncreatableType<emoji::Emoji>(
-      "im.nheko.EmojiModel", 1, 0, "Emoji", "Used by emoji models");
-    qmlRegisterUncreatableMetaObject(
-      emoji::staticMetaObject, "im.nheko.EmojiModel", 1, 0, "EmojiCategory", "Error: Only enums");
-
-    qmlRegisterType<RoomDirectoryModel>("im.nheko", 1, 0, "RoomDirectoryModel");
-
-#ifdef USE_QUICK_VIEW
-    view      = new QQuickView(parent);
-    container = QWidget::createWindowContainer(view, parent);
-#else
-    view      = new QQuickWidget(parent);
-    container = view;
-    view->setResizeMode(QQuickWidget::SizeRootObjectToView);
-    container->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
-
-    connect(view, &QQuickWidget::statusChanged, this, [](QQuickWidget::Status status) {
-        nhlog::ui()->debug("Status changed to {}", status);
-    });
-#endif
-    container->setMinimumSize(200, 200);
     updateColorPalette();
-    view->engine()->addImageProvider("MxcImage", imgProvider);
-    view->engine()->addImageProvider("colorimage", colorImgProvider);
-    view->engine()->addImageProvider("blurhash", blurhashProvider);
-    if (JdenticonProvider::isAvailable())
-        view->engine()->addImageProvider("jdenticon", jdenticonProvider);
-    view->setSource(QUrl("qrc:///qml/Root.qml"));
 
-    connect(parent, &ChatPage::themeChanged, this, &TimelineViewManager::updateColorPalette);
-    connect(dynamic_cast<ChatPage *>(parent),
+    connect(UserSettings::instance().get(),
+            &UserSettings::themeChanged,
+            this,
+            &TimelineViewManager::updateColorPalette);
+    connect(parent,
             &ChatPage::receivedRoomDeviceVerificationRequest,
             verificationManager_,
             &VerificationManager::receivedRoomDeviceVerificationRequest);
-    connect(dynamic_cast<ChatPage *>(parent),
+    connect(parent,
             &ChatPage::receivedDeviceVerificationRequest,
             verificationManager_,
             &VerificationManager::receivedDeviceVerificationRequest);
-    connect(dynamic_cast<ChatPage *>(parent),
+    connect(parent,
             &ChatPage::receivedDeviceVerificationStart,
             verificationManager_,
             &VerificationManager::receivedDeviceVerificationStart);
@@ -314,11 +138,24 @@ TimelineViewManager::TimelineViewManager(CallManager *callManager, ChatPage *par
         isInitialSync_ = true;
         emit initialSyncChanged(true);
     });
-
-    connect(this,
-            &TimelineViewManager::openImageOverlayInternalCb,
+    connect(qobject_cast<QApplication *>(QApplication::instance()),
+            &QApplication::focusWindowChanged,
             this,
-            &TimelineViewManager::openImageOverlayInternal);
+            &TimelineViewManager::focusChanged);
+    connect(parent, &ChatPage::connectionLost, this, [this] {
+        isConnected_ = false;
+        emit isConnectedChanged(false);
+    });
+    connect(parent, &ChatPage::connectionRestored, this, [this] {
+        isConnected_ = true;
+        emit isConnectedChanged(true);
+    });
+}
+
+bool
+TimelineViewManager::isWindowFocused() const
+{
+    return MainWindow::instance() == QApplication::focusWindow();
 }
 
 void
@@ -326,28 +163,31 @@ TimelineViewManager::openRoomMembers(TimelineModel *room)
 {
     if (!room)
         return;
-    MemberList *memberList = new MemberList(room->roomId(), this);
+    MemberList *memberList = new MemberList(room->roomId());
+    QQmlEngine::setObjectOwnership(memberList, QQmlEngine::JavaScriptOwnership);
     emit openRoomMembersDialog(memberList, room);
 }
 
 void
 TimelineViewManager::openRoomSettings(QString room_id)
 {
-    RoomSettings *settings = new RoomSettings(room_id, this);
+    RoomSettings *settings = new RoomSettings(room_id);
     connect(rooms_->getRoomById(room_id).data(),
             &TimelineModel::roomAvatarUrlChanged,
             settings,
             &RoomSettings::avatarChanged);
+    QQmlEngine::setObjectOwnership(settings, QQmlEngine::JavaScriptOwnership);
     emit openRoomSettingsDialog(settings);
 }
 
 void
 TimelineViewManager::openInviteUsers(QString roomId)
 {
-    InviteesModel *model = new InviteesModel{this};
+    InviteesModel *model = new InviteesModel{};
     connect(model, &InviteesModel::accept, this, [this, model, roomId]() {
         emit inviteUsers(roomId, model->mxids());
     });
+    QQmlEngine::setObjectOwnership(model, QQmlEngine::JavaScriptOwnership);
     emit openInviteUsersDialog(model);
 }
 
@@ -355,6 +195,7 @@ void
 TimelineViewManager::openGlobalUserProfile(QString userId)
 {
     UserProfile *profile = new UserProfile{QString{}, userId, this};
+    QQmlEngine::setObjectOwnership(profile, QQmlEngine::JavaScriptOwnership);
     emit openProfile(profile);
 }
 
@@ -362,7 +203,8 @@ void
 TimelineViewManager::setVideoCallItem()
 {
     WebRTCSession::instance().setVideoItem(
-      view->rootObject()->findChild<QQuickItem *>("videoCallItem"));
+      MainWindow::instance()->rootObject()->findChild<QQuickItem *>(
+        QStringLiteral("videoCallItem")));
 }
 
 void
@@ -370,6 +212,7 @@ TimelineViewManager::sync(const mtx::responses::Sync &sync_)
 {
     this->rooms_->sync(sync_);
     this->communities_->sync(sync_);
+    this->presenceEmitter->sync(sync_.presence);
 
     if (isInitialSync_) {
         this->isInitialSync_ = false;
@@ -383,7 +226,7 @@ TimelineViewManager::showEvent(const QString &room_id, const QString &event_id)
     if (auto room = rooms_->getRoomById(room_id)) {
         if (rooms_->currentRoom() != room) {
             rooms_->setCurrentRoom(room_id);
-            container->setFocus();
+            MainWindow::instance()->requestActivate();
             nhlog::ui()->info("Activated room {}", room_id.toStdString());
         }
 
@@ -398,50 +241,65 @@ TimelineViewManager::escapeEmoji(QString str) const
 }
 
 void
-TimelineViewManager::openImageOverlay(QString mxcUrl, QString eventId)
+TimelineViewManager::openImageOverlay(TimelineModel *room, QString mxcUrl, QString eventId)
 {
     if (mxcUrl.isEmpty()) {
         return;
     }
 
-    MxcImageProvider::download(
-      mxcUrl.remove("mxc://"), QSize(), [this, eventId](QString, QSize, QImage img, QString) {
-          if (img.isNull()) {
-              nhlog::ui()->error("Error when retrieving image for overlay.");
-              return;
-          }
-
-          emit openImageOverlayInternalCb(eventId, std::move(img));
-      });
+    emit showImageOverlay(room, eventId, mxcUrl);
 }
 
 void
 TimelineViewManager::openImagePackSettings(QString roomid)
 {
-    auto room = rooms_->getRoomById(roomid).get();
-    emit showImagePackSettings(room, new ImagePackListModel(roomid.toStdString(), this));
+    auto room  = rooms_->getRoomById(roomid).get();
+    auto model = new ImagePackListModel(roomid.toStdString());
+    QQmlEngine::setObjectOwnership(model, QQmlEngine::JavaScriptOwnership);
+    emit showImagePackSettings(room, model);
 }
 
 void
-TimelineViewManager::openImageOverlayInternal(QString eventId, QImage img)
+TimelineViewManager::saveMedia(QString mxcUrl)
 {
-    auto pixmap = QPixmap::fromImage(img);
+    const QString downloadsFolder =
+      QStandardPaths::writableLocation(QStandardPaths::DownloadLocation);
+    const QString openLocation = downloadsFolder + "/" + mxcUrl.splitRef(u'/').constLast();
 
-    auto imgDialog = new dialogs::ImageOverlay(pixmap);
-    imgDialog->showFullScreen();
+    const QString filename = QFileDialog::getSaveFileName(nullptr, {}, openLocation);
 
-    auto room = rooms_->currentRoom();
-    connect(imgDialog, &dialogs::ImageOverlay::saving, room, [eventId, imgDialog, room]() {
-        // hide the overlay while presenting the save dialog for better
-        // cross platform support.
-        imgDialog->hide();
+    if (filename.isEmpty())
+        return;
 
-        if (!room->saveMedia(eventId)) {
-            imgDialog->show();
-        } else {
-            imgDialog->close();
-        }
-    });
+    const auto url = mxcUrl.toStdString();
+
+    http::client()->download(url,
+                             [filename, url](const std::string &data,
+                                             const std::string &,
+                                             const std::string &,
+                                             mtx::http::RequestErr err) {
+                                 if (err) {
+                                     nhlog::net()->warn("failed to retrieve image {}: {} {}",
+                                                        url,
+                                                        err->matrix_error.error,
+                                                        static_cast<int>(err->status_code));
+                                     return;
+                                 }
+
+                                 try {
+                                     QFile file(filename);
+
+                                     if (!file.open(QIODevice::WriteOnly))
+                                         return;
+
+                                     file.write(QByteArray(data.data(), (int)data.size()));
+                                     file.close();
+
+                                     return;
+                                 } catch (const std::exception &e) {
+                                     nhlog::ui()->warn("Error while saving file to: {}", e.what());
+                                 }
+                             });
 }
 
 void
@@ -520,46 +378,45 @@ TimelineViewManager::focusMessageInput()
 QObject *
 TimelineViewManager::completerFor(QString completerName, QString roomId)
 {
-    if (completerName == "user") {
+    if (completerName == QLatin1String("user")) {
         auto userModel = new UsersModel(roomId.toStdString());
         auto proxy     = new CompletionProxyModel(userModel);
         userModel->setParent(proxy);
         return proxy;
-    } else if (completerName == "emoji") {
+    } else if (completerName == QLatin1String("emoji")) {
         auto emojiModel = new emoji::EmojiModel();
         auto proxy      = new CompletionProxyModel(emojiModel);
         emojiModel->setParent(proxy);
         return proxy;
-    } else if (completerName == "allemoji") {
+    } else if (completerName == QLatin1String("allemoji")) {
         auto emojiModel = new emoji::EmojiModel();
         auto proxy      = new CompletionProxyModel(emojiModel, 1, static_cast<size_t>(-1) / 4);
         emojiModel->setParent(proxy);
         return proxy;
-    } else if (completerName == "room") {
+    } else if (completerName == QLatin1String("room")) {
         auto roomModel = new RoomsModel(false);
         auto proxy     = new CompletionProxyModel(roomModel, 4);
         roomModel->setParent(proxy);
         return proxy;
-    } else if (completerName == "roomAliases") {
+    } else if (completerName == QLatin1String("roomAliases")) {
         auto roomModel = new RoomsModel(true);
         auto proxy     = new CompletionProxyModel(roomModel);
         roomModel->setParent(proxy);
         return proxy;
-    } else if (completerName == "stickers") {
+    } else if (completerName == QLatin1String("stickers")) {
         auto stickerModel = new CombinedImagePackModel(roomId.toStdString(), true);
         auto proxy        = new CompletionProxyModel(stickerModel, 1, static_cast<size_t>(-1) / 4);
         stickerModel->setParent(proxy);
         return proxy;
+    } else if (completerName == QLatin1String("customEmoji")) {
+        auto stickerModel = new CombinedImagePackModel(roomId.toStdString(), false);
+        auto proxy        = new CompletionProxyModel(stickerModel);
+        stickerModel->setParent(proxy);
+        return proxy;
     }
     return nullptr;
 }
 
-void
-TimelineViewManager::focusTimeline()
-{
-    getWidget()->setFocus();
-}
-
 void
 TimelineViewManager::forwardMessageToRoom(mtx::events::collections::TimelineEvents *e,
                                           QString roomId)
