@@ -1,4 +1,5 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
+// SPDX-FileCopyrightText: 2022 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -7,8 +8,6 @@
 #include <QHash>
 #include <QQuickItem>
 #include <QQuickTextDocument>
-#include <QQuickView>
-#include <QQuickWidget>
 #include <QWidget>
 
 #include <mtx/common.hpp>
@@ -24,6 +23,7 @@
 #include "emoji/Provider.h"
 #include "encryption/VerificationManager.h"
 #include "timeline/CommunitiesModel.h"
+#include "timeline/PresenceEmitter.h"
 #include "timeline/RoomlistModel.h"
 #include "voip/CallManager.h"
 #include "voip/WebRTCSession.h"
@@ -41,32 +41,28 @@ class TimelineViewManager : public QObject
 
     Q_PROPERTY(
       bool isInitialSync MEMBER isInitialSync_ READ isInitialSync NOTIFY initialSyncChanged)
-    Q_PROPERTY(
-      bool isWindowFocused MEMBER isWindowFocused_ READ isWindowFocused NOTIFY focusChanged)
+    Q_PROPERTY(bool isWindowFocused READ isWindowFocused NOTIFY focusChanged)
+    Q_PROPERTY(bool isConnected READ isConnected NOTIFY isConnectedChanged)
 
 public:
     TimelineViewManager(CallManager *callManager, ChatPage *parent = nullptr);
-    QWidget *getWidget() const { return container; }
 
     void sync(const mtx::responses::Sync &sync_);
 
-    MxcImageProvider *imageProvider() { return imgProvider; }
-    CallManager *callManager() { return callManager_; }
     VerificationManager *verificationManager() { return verificationManager_; }
 
     void clearAll() { rooms_->clear(); }
 
     Q_INVOKABLE bool isInitialSync() const { return isInitialSync_; }
-    bool isWindowFocused() const { return isWindowFocused_; }
-    Q_INVOKABLE void openImageOverlay(QString mxcUrl, QString eventId);
+    bool isWindowFocused() const;
+    bool isConnected() const { return isConnected_; }
+    Q_INVOKABLE void openImageOverlay(TimelineModel *room, QString mxcUrl, QString eventId);
     Q_INVOKABLE void openImagePackSettings(QString roomid);
+    Q_INVOKABLE void saveMedia(QString mxcUrl);
     Q_INVOKABLE QColor userColor(QString id, QColor background);
     Q_INVOKABLE QString escapeEmoji(QString str) const;
     Q_INVOKABLE QString htmlEscape(QString str) const { return str.toHtmlEscaped(); }
 
-    Q_INVOKABLE QString userPresence(QString id) const;
-    Q_INVOKABLE QString userStatus(QString id) const;
-
     Q_INVOKABLE void openRoomMembers(TimelineModel *room);
     Q_INVOKABLE void openRoomSettings(QString room_id);
     Q_INVOKABLE void openInviteUsers(QString roomId);
@@ -79,6 +75,7 @@ public:
 signals:
     void activeTimelineChanged(TimelineModel *timeline);
     void initialSyncChanged(bool isInitialSync);
+    void isConnectedChanged(bool state);
     void replyingEventChanged(QString replyingEvent);
     void replyClosed();
     void inviteUsers(QString roomId, QStringList users);
@@ -86,26 +83,20 @@ signals:
     void narrowViewChanged();
     void focusChanged();
     void focusInput();
-    void openImageOverlayInternalCb(QString eventId, QImage img);
     void openRoomMembersDialog(MemberList *members, TimelineModel *room);
     void openRoomSettingsDialog(RoomSettings *settings);
     void openInviteUsersDialog(InviteesModel *invitees);
     void openProfile(UserProfile *profile);
     void showImagePackSettings(TimelineModel *room, ImagePackListModel *packlist);
     void openLeaveRoomDialog(QString roomid);
+    void showImageOverlay(TimelineModel *room, QString eventId, QString url);
 
 public slots:
     void updateReadReceipts(const QString &room_id, const std::vector<QString> &event_ids);
     void receivedSessionKey(const std::string &room_id, const std::string &session_id);
     void initializeRoomlist();
-    void chatFocusChanged(bool focused)
-    {
-        isWindowFocused_ = focused;
-        emit focusChanged();
-    }
 
     void showEvent(const QString &room_id, const QString &event_id);
-    void focusTimeline();
 
     void updateColorPalette();
     void queueReply(const QString &roomid, const QString &repliedToEvent, const QString &replyBody);
@@ -116,36 +107,21 @@ public slots:
 
     void setVideoCallItem();
 
-    QObject *completerFor(QString completerName, QString roomId = "");
+    QObject *completerFor(QString completerName, QString roomId = QLatin1String(QLatin1String("")));
     void forwardMessageToRoom(mtx::events::collections::TimelineEvents *e, QString roomId);
 
     RoomlistModel *rooms() { return rooms_; }
 
-private slots:
-    void openImageOverlayInternal(QString eventId, QImage img);
-
 private:
-#ifdef USE_QUICK_VIEW
-    QQuickView *view;
-#else
-    QQuickWidget *view;
-#endif
-    QWidget *container;
-
-    MxcImageProvider *imgProvider;
-    ColorImageProvider *colorImgProvider;
-    BlurhashProvider *blurhashProvider;
-    JdenticonProvider *jdenticonProvider;
-
-    bool isInitialSync_   = true;
-    bool isWindowFocused_ = false;
+    bool isInitialSync_ = true;
+    bool isConnected_   = true;
 
     RoomlistModel *rooms_          = nullptr;
     CommunitiesModel *communities_ = nullptr;
 
     // don't move this above the rooms_
-    CallManager *callManager_                 = nullptr;
     VerificationManager *verificationManager_ = nullptr;
+    PresenceEmitter *presenceEmitter          = nullptr;
 
     QHash<QPair<QString, quint64>, QColor> userColors;
 };
