@@ -1,4 +1,5 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
+// SPDX-FileCopyrightText: 2022 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -11,8 +12,6 @@
 #include "Theme.h"
 #include "UserProfile.h"
 
-class QWindow;
-
 class Nheko : public QObject
 {
     Q_OBJECT
@@ -24,6 +23,7 @@ class Nheko : public QObject
     Q_PROPERTY(int paddingSmall READ paddingSmall CONSTANT)
     Q_PROPERTY(int paddingMedium READ paddingMedium CONSTANT)
     Q_PROPERTY(int paddingLarge READ paddingLarge CONSTANT)
+    Q_PROPERTY(int tooltipDelay READ tooltipDelay CONSTANT)
 
     Q_PROPERTY(UserProfile *currentUser READ currentUser NOTIFY profileChanged)
 
@@ -39,6 +39,9 @@ public:
     int paddingSmall() const { return 4; }
     int paddingMedium() const { return 8; }
     int paddingLarge() const { return 20; }
+
+    int tooltipDelay() const;
+
     UserProfile *currentUser() const;
 
     Q_INVOKABLE QFont monospaceFont() const
@@ -50,7 +53,6 @@ public:
     Q_INVOKABLE void showUserSettingsPage() const;
     Q_INVOKABLE void logout() const;
     Q_INVOKABLE void openCreateRoomDialog() const;
-    Q_INVOKABLE void reparent(QWindow *win) const;
 
 public slots:
     void updateUserProfile();
