@@ -1,4 +1,5 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
+// SPDX-FileCopyrightText: 2022 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -11,10 +12,13 @@
 #include <QMimeDatabase>
 #include <QStandardPaths>
 #include <QVBoxLayout>
+#include <mtx/events/event_type.hpp>
 #include <mtx/responses/common.hpp>
 #include <mtx/responses/media.hpp>
+#include <mtxclient/http/client.hpp>
 
 #include "Cache.h"
+#include "Cache_p.h"
 #include "Config.h"
 #include "Logging.h"
 #include "MatrixClient.h"
@@ -234,8 +238,10 @@ RoomSettings::roomName() const
 QString
 RoomSettings::roomTopic() const
 {
-    return utils::replaceEmoji(utils::linkifyMessage(
-      QString::fromStdString(info_.topic).toHtmlEscaped().replace("\n", "<br>")));
+    return utils::replaceEmoji(
+      utils::linkifyMessage(QString::fromStdString(info_.topic)
+                              .toHtmlEscaped()
+                              .replace(QLatin1String("\n"), QLatin1String("<br>"))));
 }
 
 QString
@@ -563,10 +569,10 @@ RoomSettings::updateAvatar()
     QMimeDatabase db;
     QMimeType mime = db.mimeTypeForFile(fileName, QMimeDatabase::MatchContent);
 
-    const auto format = mime.name().split("/")[0];
+    const auto format = mime.name().split(QStringLiteral("/"))[0];
 
     QFile file{fileName, this};
-    if (format != "image") {
+    if (format != QLatin1String("image")) {
         emit displayError(tr("The selected file is not an image"));
         return;
     }
