@@ -1,4 +1,5 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
+// SPDX-FileCopyrightText: 2022 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -40,6 +41,7 @@ void
 TextField::setBackgroundColor(const QColor &color)
 {
     background_color_ = color;
+    emit backgroundColorChanged();
 }
 
 QColor
@@ -130,6 +132,7 @@ void
 TextField::setLabelColor(const QColor &color)
 {
     label_color_ = color;
+    emit labelColorChanged();
     update();
 }
 
@@ -147,6 +150,7 @@ void
 TextField::setInkColor(const QColor &color)
 {
     ink_color_ = color;
+    emit inkColorChanged();
     update();
 }
 
@@ -164,6 +168,7 @@ void
 TextField::setUnderlineColor(const QColor &color)
 {
     underline_color_ = color;
+    emit underlineColorChanged();
     update();
 }
 
