@@ -1,4 +1,5 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
+// SPDX-FileCopyrightText: 2022 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -69,6 +70,7 @@ Toggle::setDisabledColor(const QColor &color)
 {
     disabledColor_ = color;
     setupProperties();
+    emit disabledColorChanged();
 }
 
 void
@@ -76,6 +78,7 @@ Toggle::setActiveColor(const QColor &color)
 {
     activeColor_ = color;
     setupProperties();
+    emit activeColorChanged();
 }
 
 void
@@ -83,6 +86,7 @@ Toggle::setInactiveColor(const QColor &color)
 {
     inactiveColor_ = color;
     setupProperties();
+    emit inactiveColorChanged();
 }
 
 void
@@ -90,6 +94,7 @@ Toggle::setTrackColor(const QColor &color)
 {
     trackColor_ = color;
     setupProperties();
+    emit trackColorChanged();
 }
 
 ToggleThumb::ToggleThumb(Toggle *parent)
@@ -174,6 +179,7 @@ void
 ToggleTrack::setTrackColor(const QColor &color)
 {
     trackColor_ = color;
+    emit trackColorChanged();
     update();
 }
 
