@@ -1,4 +1,5 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
+// SPDX-FileCopyrightText: 2022 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -28,7 +29,7 @@ UserProfile::UserProfile(QString roomid,
   , manager(manager_)
   , model(parent)
 {
-    globalAvatarUrl = "";
+    globalAvatarUrl = QLatin1String("");
 
     connect(this,
             &UserProfile::globalUsernameRetrieved,
@@ -124,7 +125,7 @@ UserProfile::avatarUrl()
 bool
 UserProfile::isGlobalUserProfile() const
 {
-    return roomid_ == "";
+    return roomid_ == QLatin1String("");
 }
 
 crypto::Trust
@@ -175,8 +176,6 @@ UserProfile::refreshDevices()
 void
 UserProfile::fetchDeviceList(const QString &userID)
 {
-    auto localUser = utils::localUser();
-
     if (!cache::client() || !cache::client()->isDatabaseReady())
         return;
 
@@ -234,6 +233,7 @@ UserProfile::updateVerificationStatus()
     this->isUserVerified = verificationStatus.user_verified;
     emit userStatusChanged();
 
+    deviceInfo.reserve(devices.size());
     for (const auto &d : devices) {
         auto device = d.second;
         verification::Status verified =
@@ -246,9 +246,9 @@ UserProfile::updateVerificationStatus()
         if (isSelf() && device.device_id == ::http::client()->device_id())
             verified = verification::Status::SELF;
 
-        deviceInfo.push_back({QString::fromStdString(d.first),
-                              QString::fromStdString(device.unsigned_info.device_display_name),
-                              verified});
+        deviceInfo.emplace_back(QString::fromStdString(d.first),
+                                QString::fromStdString(device.unsigned_info.device_display_name),
+                                verified);
     }
 
     // For self, also query devices without keys
@@ -272,17 +272,17 @@ UserProfile::updateVerificationStatus()
                           found = true;
                           // Gottem! Let's fill in the blanks
                           e.lastIp = QString::fromStdString(d.last_seen_ip);
-                          e.lastTs = d.last_seen_ts;
+                          e.lastTs = static_cast<qlonglong>(d.last_seen_ts);
                           break;
                       }
                   }
                   // No entry? Let's add one.
                   if (!found) {
-                      deviceInfo.push_back({QString::fromStdString(d.device_id),
-                                            QString::fromStdString(d.display_name),
-                                            verification::NOT_APPLICABLE,
-                                            QString::fromStdString(d.last_seen_ip),
-                                            d.last_seen_ts});
+                      deviceInfo.emplace_back(QString::fromStdString(d.device_id),
+                                              QString::fromStdString(d.display_name),
+                                              verification::NOT_APPLICABLE,
+                                              QString::fromStdString(d.last_seen_ip),
+                                              d.last_seen_ts);
                   }
               }
 
@@ -299,7 +299,7 @@ UserProfile::updateVerificationStatus()
 void
 UserProfile::banUser()
 {
-    ChatPage::instance()->banUser(this->userid_, "");
+    ChatPage::instance()->banUser(this->userid_, QLatin1String(""));
 }
 
 // void ignoreUser(){
@@ -309,7 +309,7 @@ UserProfile::banUser()
 void
 UserProfile::kickUser()
 {
-    ChatPage::instance()->kickUser(this->userid_, "");
+    ChatPage::instance()->kickUser(this->userid_, QLatin1String(""));
 }
 
 void
@@ -392,10 +392,10 @@ UserProfile::changeAvatar()
     QMimeDatabase db;
     QMimeType mime = db.mimeTypeForFile(fileName, QMimeDatabase::MatchContent);
 
-    const auto format = mime.name().split("/")[0];
+    const auto format = mime.name().split(QStringLiteral("/"))[0];
 
     QFile file{fileName, this};
-    if (format != "image") {
+    if (format != QLatin1String("image")) {
         emit displayError(tr("The selected file is not an image"));
         return;
     }
