@@ -1,4 +1,5 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
+// SPDX-FileCopyrightText: 2022 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -12,6 +13,7 @@
 #include <mtx/responses/common.hpp>
 
 #include "CacheCryptoStructs.h"
+#include "timeline/TimelineModel.h"
 
 namespace verification {
 Q_NAMESPACE
@@ -28,7 +30,6 @@ Q_ENUM_NS(Status)
 }
 
 class DeviceVerificationFlow;
-class TimelineModel;
 class TimelineViewManager;
 
 class DeviceInfo
@@ -133,8 +134,8 @@ public:
     bool isLoading() const;
     TimelineModel *room() const { return model; }
 
-    Q_INVOKABLE void verify(QString device = "");
-    Q_INVOKABLE void unverify(QString device = "");
+    Q_INVOKABLE void verify(QString device = QLatin1String(""));
+    Q_INVOKABLE void unverify(QString device = QLatin1String(""));
     Q_INVOKABLE void fetchDeviceList(const QString &userID);
     Q_INVOKABLE void refreshDevices();
     Q_INVOKABLE void banUser();
