@@ -1,4 +1,5 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
+// SPDX-FileCopyrightText: 2022 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -70,7 +71,8 @@ CallManager::CallManager(QObject *parent)
           QTimer::singleShot(timeoutms_, this, [this, callid]() {
               if (session_.state() == webrtc::State::OFFERSENT && callid == callid_) {
                   hangUp(CallHangUp::Reason::InviteTimeOut);
-                  emit ChatPage::instance()->showNotification("The remote side failed to pick up.");
+                  emit ChatPage::instance()->showNotification(
+                    QStringLiteral("The remote side failed to pick up."));
               }
           });
       });
@@ -115,13 +117,13 @@ CallManager::CallManager(QObject *parent)
     connect(&session_, &WebRTCSession::stateChanged, this, [this](webrtc::State state) {
         switch (state) {
         case webrtc::State::DISCONNECTED:
-            playRingtone(QUrl("qrc:/media/media/callend.ogg"), false);
+            playRingtone(QUrl(QStringLiteral("qrc:/media/media/callend.ogg")), false);
             clear();
             break;
         case webrtc::State::ICEFAILED: {
-            QString error("Call connection failed.");
+            QString error(QStringLiteral("Call connection failed."));
             if (turnURIs_.empty())
-                error += " Your homeserver has no configured TURN server.";
+                error += QLatin1String(" Your homeserver has no configured TURN server.");
             emit ChatPage::instance()->showNotification(error);
             hangUp(CallHangUp::Reason::ICEFailed);
             break;
@@ -143,6 +145,7 @@ CallManager::CallManager(QObject *parent)
 
     connect(&player_,
             QOverload<QMediaPlayer::Error>::of(&QMediaPlayer::error),
+            this,
             [this](QMediaPlayer::Error error) {
                 stopRingtone();
                 switch (error) {
@@ -176,7 +179,8 @@ CallManager::sendInvite(const QString &roomid, CallType callType, unsigned int w
 
     auto roomInfo = cache::singleRoomInfo(roomid.toStdString());
     if (roomInfo.member_count != 2) {
-        emit ChatPage::instance()->showNotification("Calls are limited to 1:1 rooms.");
+        emit ChatPage::instance()->showNotification(
+          QStringLiteral("Calls are limited to 1:1 rooms."));
         return;
     }
 
@@ -202,10 +206,10 @@ CallManager::sendInvite(const QString &roomid, CallType callType, unsigned int w
     callPartyDisplayName_ = callee.display_name.isEmpty() ? callee.user_id : callee.display_name;
     callPartyAvatarUrl_   = QString::fromStdString(roomInfo.avatar_url);
     emit newInviteState();
-    playRingtone(QUrl("qrc:/media/media/ringback.ogg"), true);
+    playRingtone(QUrl(QStringLiteral("qrc:/media/media/ringback.ogg")), true);
     if (!session_.createOffer(callType,
                               callType == CallType::SCREEN ? windows_[windowIndex].second : 0)) {
-        emit ChatPage::instance()->showNotification("Problem setting up call.");
+        emit ChatPage::instance()->showNotification(QStringLiteral("Problem setting up call."));
         endCall();
     }
 }
@@ -288,10 +292,11 @@ CallManager::handleEvent(const RoomEvent<CallInvite> &callInviteEvent)
         return;
     }
 
-    const QString &ringtone = ChatPage::instance()->userSettings()->ringtone();
-    if (ringtone != "Mute")
-        playRingtone(ringtone == "Default" ? QUrl("qrc:/media/media/ring.ogg")
-                                           : QUrl::fromLocalFile(ringtone),
+    const QString &ringtone = UserSettings::instance()->ringtone();
+    if (ringtone != QLatin1String("Mute"))
+        playRingtone(ringtone == QLatin1String("Default")
+                       ? QUrl(QStringLiteral("qrc:/media/media/ring.ogg"))
+                       : QUrl::fromLocalFile(ringtone),
                      true);
     roomid_ = QString::fromStdString(callInviteEvent.room_id);
     callid_ = callInviteEvent.content.call_id;
@@ -327,7 +332,7 @@ CallManager::acceptInvite()
 
     session_.setTurnServers(turnURIs_);
     if (!session_.acceptOffer(inviteSDP_)) {
-        emit ChatPage::instance()->showNotification("Problem setting up call.");
+        emit ChatPage::instance()->showNotification(QStringLiteral("Problem setting up call."));
         hangUp();
         return;
     }
@@ -369,7 +374,8 @@ CallManager::handleEvent(const RoomEvent<CallAnswer> &callAnswerEvent)
     if (callAnswerEvent.sender == utils::localUser().toStdString() &&
         callid_ == callAnswerEvent.content.call_id) {
         if (!isOnCall()) {
-            emit ChatPage::instance()->showNotification("Call answered on another device.");
+            emit ChatPage::instance()->showNotification(
+              QStringLiteral("Call answered on another device."));
             stopRingtone();
             haveCallInvite_ = false;
             emit newInviteState();
@@ -380,7 +386,7 @@ CallManager::handleEvent(const RoomEvent<CallAnswer> &callAnswerEvent)
     if (isOnCall() && callid_ == callAnswerEvent.content.call_id) {
         stopRingtone();
         if (!session_.acceptAnswer(callAnswerEvent.content.sdp)) {
-            emit ChatPage::instance()->showNotification("Problem setting up call.");
+            emit ChatPage::instance()->showNotification(QStringLiteral("Problem setting up call."));
             hangUp();
         }
     }
@@ -425,8 +431,8 @@ QStringList
 CallManager::devices(bool isVideo) const
 {
     QStringList ret;
-    const QString &defaultDevice = isVideo ? ChatPage::instance()->userSettings()->camera()
-                                           : ChatPage::instance()->userSettings()->microphone();
+    const QString &defaultDevice =
+      isVideo ? UserSettings::instance()->camera() : UserSettings::instance()->microphone();
     std::vector<std::string> devices =
       CallDevices::instance().names(isVideo, defaultDevice.toStdString());
     ret.reserve(devices.size());
